/* @compact */

/* Copyright (C) MOR YAZILIM. http://www.moryazilim.com
 * All rights reserved. This software is published under the terms of the
 * GNU Lesser General Public License. http://www.gnu.org/licenses/lgpl.txt
 */


/**
 * @fileoverview
 * Contains SuggestBox and SuggestBoxManager classes
 * @version $Id: SuggestBox.js 226 2008-10-06 14:51:47Z erkan $
 */

/**
 * @class
 * Represents a very special Process Field called SuggestBox.
 * A SuggestBox is a highly customizable and powerful MoreMotion Object that suggests options to the user
 * as he types in the <code>TEXT INPUT</code> field. After the predefined idle time is expired, this class
 * prepares an {@link AjaxRequest} to retrieve the data from the MoreMotion Ajax Service to suggest to the user.
 * A Drop-down box is built to display the options received.
 * <p>This class wraps two HTML elements which are a <code>&LT;INPUT TYPE="TEXT"&GT;</code> element and a
 * <code>&LT;INPUT TYPE="HIDDEN"&GT;</code> element. The <code>TEXT INPUT</code> element is for entering new value or filtering the
 * options to be suggested and the <code>HIDDEN INPUT</code> element is to keep the value that corresponds to the selected option.
 * Depending on the specified properties, this class ensures that the user selects an existing option
 * or types in a non-existing value.<p>
 * <b>Handlers:</b><br>
 * SuggestBox event handler functions that are registered for the SuggestBox events must have one parameter
 * which is the SuggestBox object.<br><code><br>
 * function MyOnBeforeRowPaintHandler(sbox) {<br>
 * &nbsp;&nbsp;if (sbox.getFieldValue("DISABLED") == "false") {<br>
 * &nbsp;&nbsp;&nbsp;&nbsp;...<br>
 * &nbsp;&nbsp;}<br>
 * }</code>
 *
 *
 * @extends ProcessField
 *
 */
function SuggestBox(node) {

  /**
   * SuggestBox Properties Object.
   *
   * <br/><br/><table border="1" style="border-collapse: collapse" cellspacing="3">
   * <tr><td><b>valueFieldName</b></td><td width="654">The name the <code>HIDDEN INPUT</code> field that
   * will keep the value associated with the selected option.</td>
   * <tr><td><b>actionName</b></td> <td width="654">The name of the Ajax actionParams configuration element </td></tr>
   * <tr><td valign="top"><b>autoComplete</b></td> <td width="654">AutoComplete Flag. <code>true</code>
   * means as the user navigates on the options on the DropDownBox the value of focused options
   * will be transfered to the <code>TEXT INPUT</code> field.</td></tr>
   * <tr><td><b>delay</b></td><td width="654">The time (in miliseconds) to elapse after the last time
   * the user pressed a key.</td></tr>
   * <tr><td valign="top"><b>align</b></td><td width="654">The horizontal alignment of the DropDownBox relative to the <code>TEXT INPUT</code> field
   * The values are "left" (the default), "right" and "center"</td></tr>
   * <tr><td><b>elmWidth</b></td><td width="654">The width of the <code>TEXT INPUT</code> field in pixels </td></tr>
   * <tr><td><b>width</b></td><td width="654">The width of the DropDownBox in pixels</td></tr>
   * <tr><td><b>height</b></td><td width="654"> The height of the <code>INPUT</code> field in pixels</td></tr>
   * <tr><td valign="top"><b>suggestField</b></td><td width="654"> The name of the field in the response whose value will be assigned as the option text (the suggested value)
   * of this SuggestBox</td></tr>
   * <tr><td><b>valueField</b></td><td width="654">The name of the field in the response whose value will be assigned as the value of this SuggestBox </td></tr>
   * <tr><td><b>waitForResponse</b></td><td width="654"> Flag to wait for the Ajax Response once it is sent to the Ajax Service. (Not Recommended)</td></tr>
   * <tr><td><b>boxStyle</b></td><td width="654">The CSS Styles for the DropDownBox area.</td></tr>
   * <tr><td><b>hdrStyle</b></td><td width="654">The CSS Styles for the rows header.</td></tr>
   * <tr><td><b>hdrCaptions</b></td><td width="654">The header captions (separated with commas) </td></tr>
   * <tr><td><b>hdrStyles</b></td><td width="654">The CSS Styles for header columns. They should be separated with "|" characters. </td></tr>
   * <tr><td><b>rowStyle</b></td><td width="654">The CSS Styles for the DropDownBox Rows.
   * The styles for the focussed rows can be given after a "/" character. e.g. <code>rowStyle="background-color:silver/white; color:navy/black". </code></td></tr>
   * <tr><td><b>colFields</b></td><td width="654"> The Result set field names (separated with commas) for the Row Columns.</td></tr>
   * <tr><td><b>colWidths</b></td><td width="654">The Widths in percent (separated with commas) of the Row Columns. </td></tr>
   * <tr><td><b>colAligns</b></td><td width="654"> The Alignments (separated with commas) of the Row Columns. The values can be "left", "right", "center"</td></tr>
   * <tr><td><b>colStyles</b></td><td width="654">The CSS Styles for Row Columns. They should be separated with "|" characters. </td></tr>
   * <tr><td><b>sbarStyle</b></td><td width="654">The CSS Styles for the Status Bar.
   * The styles for the last page can be given after a "/" character. e.g. <code>rowStyle="background-color:#silver/pink;</code></td></tr>
   * <tr><td><b>onFocusHandler</b></td><td width="654">The name of the onFocus Handler Function. </td></tr>
   * <tr><td><b>onBeforeSendRequestHandler</b></td><td width="654">The name of the onBeforeSendRequest Handler Function </td></tr>
   * <tr><td><b>onReceiveResponseHandler</b></td><td width="654"> The name of the onReceiveResponse Handler Function</td></tr>
   * <tr><td><b>onBeforeRowPaint</b></td><td width="654"> The name of the onBeforeRowPaint Handler Function</td></tr>
   * <tr><td><b>onFocusOptionHandler</b></td><td width="654"> The name of the onFocusOption Handler Function</td></tr>
   * <tr><td><b>onChangeHandler</b></td><td width="654"> The name of the onChange Handler Function</td></tr>
   * <tr><td><b>onBlurHandler</b></td><td width="654">The name of the onBlur Handler Function </td></tr>
   * <tr><td valign="top"><b>required</b></td><td width="654"> The property that specifies what type of input is required for this SuggestBox. The options are:
   * <ol>
   * <li>No input is required</li>
   * <li>The <code>TEXT INPUT</code> must have a value</li>
   * <li>The both <code>TEXT INPUT</code> field and <code>HIDDEN INPUT</code> fields must have a value</li>.
   * This means The typed or selected option must have an associated value in the system.
   * <li>If <code>TEXT INPUT</code> field has a value then the <code>HIDDEN INPUT</code> must also have a value</li>.
   * <li>The <code>HIDDEN INPUT</code> must not have a value</li>. This means typed value must be new and therefore
   * </ol></td></tr>
   * <tr><td><b>valErrorHandler</b></td><td width="654"> The name of the Validation Error Handler Function</td></tr>
   * <tr><td><b>valHandler</b></td><td width="654"> The name of the Validation Handler Function</td></tr>
   * <tr><td><b>pageInfo</b></td><td width="654"> Page Information as String. See {@link PageInfo}.
   * This property should be supplied only if the this SuggestBox is resides in a DOM content that was fetched
   * with a {@link MoreMotionAjaxManager#refreshAreas} method call. It is required to access the configuration of the origin page.</td></tr>
   * </table>
   * <p/>
   * <u>Note:</u> &nbsp; For all the style properties, instead of CSS style strings, CSS class names, that are prefixed with a period ("."), can be specified.
   * The both of the following definitions are valid. <code> <br/>
   * rowStyle="background-color:#f1f1f1/white; font:Arial; color:navy/black"  <br/>
   * rowStyle=".row/.rowhl"
   * </code>
   */
  this.props = null;

  try {
    this.base = ProcessField;
    this.base(node);
  } catch (e) {
    this.base = MoreMotionObject;
    this.base(node);
  }

  /** @type boolean */
  this.isSuggestBox = true;

  /**
   * The handle to the <code>TEXT INPUT</code> element that is visible and editable.
   * @type HTMLElement
   */
  this.elm = null; //-<

  /**
   * The handle to the <code>HIDDEN INPUT</code> element that will keep the value associated with the selected option.
   * @type HTMLElement
   */
  this.selm = null; //-<

  /** @private */
  this.wrapElements = function() {
    var elms = this.node.parentNode.getElementsByTagName("input");
    if (elms && elms.length) {
      for (var i = 0; i < elms.length; i++) {
        var elm = elms[i];
        if (elm.name.endsWith("_" + this.name) || elm.name == this.name) {
          this.elm = elm;
        } else if (elm.name.endsWith("_" + this.props.valueFieldName) || elm.name == this.props.valueFieldName) {
          this.selm = elm;
        }
      }
    }
  };

  this.wrapElements();

  /** @private */
  this.init = function() {

    if (this.elm.readOnly) return false;

    this.keyIsDown = false;
    this.enabled = true;
    this.vlen = this.elm.value.length;
    this.prev_value = this.elm.value;
    this.pos = -1;
    this.keyCode = -1;
    this.justInitialized = true;
    this.hasFocus = true;
    this.time = -1;
    this.sendPending = false;
    this.cpos = this.elm.value.length;
    this.notifiedFocusOptionPos = null;
    this.notifiedChangedValue = null;
    this.changed = false;
    this.prevValue = this.elm.value + "~~";
    this.navigateUp = false;

    if (!this.elm.id) this.elm.id = OMgr.getUniqueId();

    return true;

  };

  this.init();

  this.focus = function() {
    this.init();
    this.runHandler("onFocusHandler");
  };

  /** Closes the DropDownBox if it is open */
  this.closeDropDownBox = function() {
    var div = this.getDiv();
    div.innerHTML = '';
    this.keyCode = '';
    ShowDropDowns(div);
  };

  /**
   * Returns
   */
  this.isDropDownBoxOpen = function() {
    var div = this.getDiv();
    return div.innerHTML != '';
  };

  /** @private */
  this.getDiv = function() {

    var divid = "sbx_divid";
    var div = document.getElementById(divid);
    if (!div) {
      div = document.createElement("div");
      div.id = divid;
      document.body.appendChild(div);
    }
    return div;
  };


  /** @private */
  this.onKeyUp = function(e,params) {

    // if (e.ctrlKey) return true;

    var keyCode = (e.keyCode) ? e.keyCode : e.keyChar;
    this.keyCode = keyCode;

    if (this.elm.readOnly) return false;

    if (this.justInitialized) {
      this.justInitialized = false;
      return false;
    }

    this.setModified();

    if (this.prevValue && (this.elm.value + "~~") != this.prevValue) {
      if (keyCode != 13 && keyCode != 38 && keyCode != 40 && keyCode != 33 && keyCode != 34) {
        // ENTER, UP, DOWN, PAGE UP, PAGE DOWN
        this.changed = true;
        try {
          this.resp = null;
        } catch (e) {}
      }
    }

    if (this.pasteCommand) {}

    else if (
        keyCode == 16 || keyCode == 17 || keyCode == 19 ||
        keyCode == 35 || keyCode == 36 || keyCode == 37 || keyCode == 39 ||
        keyCode == 45 || keyCode == 91 || keyCode == 93
       ) return true;

    else if (
        keyCode == 33 || keyCode == 34 || keyCode == 38  || keyCode == 40
       ) return false;

    else if (keyCode == 13) {
      // this.enter();
      return false;
    }

    if (this.elm.value.length == 0) {
      this.selm.value = "";
      this.prev_value = "";

      if (keyCode == 46 || keyCode == 8 ) {
        this.closeDropDownBox();
        this.valueCleared = true;
        this.len = 0;
        this.pos = -1;
        this.resp = null;
        this.pi = null;
        this.params = "";
      }
      this.valueCleared = null;

      this.time = -1;
      this.vlen = 0;
      this.cpos = 0;
      this.sendPending = false;
      return true;
    }

    if (keyCode == 46) {
      this.selm.value = "";
      this.closeDropDownBox();
      return false;
    }

    if (keyCode == 27) { // ESC pressed
      if (this.isDropDownBoxOpen()) {
        this.closeDropDownBox();
        if (!this.boolProp("autoComplete")) this.selm.value = "";
      }
      return false;
    }

    this.selm.value = "";

    this.time = (new Date()).getTime();
    this.cpos = this.elm.value.length;

    if (this.elm.value == this.prev_value) {
      return true;
    }

    this.params = params;
    this.requestParams = new RequestParameterAccumulator(params);
    this.requestParams.add("_cfg",this.props.actionName);

    if (!this.sendPending) {
      this.sendPending = true;
      this.selm.value = "";
      setTimeout('SBoxMgr.sendRequest("' + this.elm.id + '")',this.numberProp("delay",500));
    }

    return true;
  };


  /*
  this.comboAction = function(params) {

    this.elm.value = "";
    this.selm.value = "";

    this.time = (new Date()).getTime();

    this.cpos = this.elm.value.length;

    this.requestParams = new RequestParameterAccumulator(params);
    this.requestParams.add("_cfg",this.props.actionName);

    if (!this.sendPending) {
      this.sendPending = true;
      this.selm.value = "";
      setTimeout('SBoxMgr.sendRequest("' + this.elm.id + '")',10);
      return true;
    }

  }; */
  
  
  /** @private */
  this.checkOnChange = function() {
    if (this.props.onChangeHandler && this.changed && this.notifiedChangedValue != this.elm.value) {
      this.runHandler("onChangeHandler");
      this.notifiedChangedValue = this.elm.value;
    }
  };

  /** @private */
  this.onKeyDown = function(e) {

    if (this.elm.readOnly) return false;

    this.justInitialized = false;

    var keyCode = (e.keyCode) ? e.keyCode : e.keyChar;
    this.keyCode = keyCode;

    this.pasteCommand = (e.ctrlKey && keyCode == 86) || (e.shiftKey && keyCode == 45);

    if (keyCode == 35 || keyCode == 36 || keyCode == 37 || keyCode == 39) {

      if (keyCode == 36) { // HOME
        this.cpos = 0;
        return true;

      } else if (keyCode == 35) { // END
        this.cpos = this.elm.value.length;
        return true;

      } else if (keyCode == 37) { // LEFT
        this.cpos--;
        if (this.cpos < 0) this.cpos = 0;
        if (e.ctrlKey) {
          this.selectRange(this.cpos);
          return false;
        }

      } else if (keyCode == 39) { // RIGHT
        this.cpos++;
        if (this.cpos > this.elm.value.length) this.cpos = this.elm.value.length;
        if (e.ctrlKey) {
          this.selectRange(this.cpos);
          return false;
        }
      }
      return true;
    }

    if (keyCode == 27) { // ESC
      return false;
    }

    else if (keyCode == 38) { // UP
      this.up();
      return false;

    } else if (keyCode == 40) { // DOWN
      this.down();
      return false;

    } else if (keyCode == 33) { // PAGE UP
      this.up(true,e.ctrlKey);
      return false;

    } else if (keyCode == 34) { // PAGE DOWN
      this.down(true,e.ctrlKey);
      return false;

    }

    else if (keyCode == 13) { // ENTER
      this.enter();
      return false;
    }

    this.vlen = this.elm.value.length;
    this.prev_value = this.elm.value;

    return true;

  };

  /** @private */
  this.enter = function() {

    this.keyIsDown = true;

    try {

      var v = this.getFieldValue(this.props.suggestField,this.pos);
      if (v) {
        this.elm.value = v;
      }
      if (this.hasFocus) this.selectRange(this.elm.value.length);

      this.vlen = this.elm.value.length;
      this.prev_value = this.elm.value;

      this.cpos = this.elm.value.length;

      var div = this.getDiv();
      div.innerHTML = '';
      ShowDropDowns(div);

    } catch(e) {}

    this.checkOnChange();
    /*
    if (this.props.onChangeHandler && this.changed && this.notifiedChangedValue != this.elm.value) {
      this.runHandler("onChangeHandler");
      this.notifiedChangedValue = this.elm.value;
    } */

  };

  /** @private */
  this.up = function(PAGE,CTRL) {

    var pos = PAGE ? -1 : this.pos - 1;
    if (pos < 0) {
      if (this.pi && this.pi.currentPage > 1) {
        this.navigateUp = true;
        SBoxMgr.navigateTo(this.elm.id, (CTRL ? 'firstPage' : 'previousPage'));
        return;
      }
      pos = 0;
    } else {
      this.select(pos);
    }
  };


  /** @private */
  this.down = function(PAGE,CTRL) {

    var len = this.len || 0;
    var pos = PAGE ? len : this.pos + 1;
    if (pos >= len) {
      pos = len - 1;
      if ((this.pi && this.pi.morePages) || !this.elm.value) {
        SBoxMgr.navigateTo(this.elm.id, (CTRL ? 'lastPage' : 'nextPage'));
        return;
      }

    } else {
      this.select(pos);
    }
  };

  /** @private */
  this.bottom = function() {
    this.select(this.len - 1);
  };

  /** @private */
  this.blur = function() {

    if (this.elm.readOnly) return false;

    this.hasFocus = false;

    if (this.sendPending) {
      // this.sendPending = false;
      // SBoxMgr.sendRequest(this.elm.id);

    } else {

      if (!this.boolProp("autoComplete") && this.isDropDownBoxOpen() && this.pos > -1) {
        this.selm.value = "";
      }

      this.checkOnChange();
      /*
      if (this.props.onChangeHandler && this.changed && this.notifiedChangedValue != this.elm.value) {
        this.runHandler("onChangeHandler");
        this.notifiedChangedValue = this.elm.value;
      } */

    }

    if (this.isDropDownBoxOpen()) {
      this.closeDropDownBox();
      if (!this.boolProp("autoComplete")) this.selm.value = "";
    }

    this.runHandler("onBlurHandler");

    return true;

  };

  /** @private */
  this._setValue = function(matched) {

    if (this.prevValue && (this.elm.value + "~~") != this.prevValue) {
      this.changed = true;
    }

    var v = this.getFieldValue(this.props.valueField,this.pos);

    if (this.boolProp("autoComplete")) {
      var s = this.getFieldValue(this.props.suggestField,this.pos);
      if (s) this.elm.value = s;
      this.selm.value = v;
    } else {
      if (matched || this.hasFocus) this.selm.value = v;
      else this.selm.value = "";
    }

    if (!this.hasFocus) {
      this.checkOnChange();
      /*
      if (this.props.onChangeHandler && this.changed && this.notifiedChangedValue != this.elm.value) {
        this.runHandler("onChangeHandler");
        this.notifiedChangedValue = this.elm.value;
      } */
    }

    this.setModified();

  };


  /** Clears the values of both <code>TEXT INPUT</code> and <code>HIDDEN INPUT</code> fields. */
  this.clear = function() {
    this.elm.value = "";
    this.selm.value = "";
  };

  /** @private */
  this.selectRange = function(start) {
    this.elm.focus();
    if (this.elm.createTextRange) {
      var range = this.elm.createTextRange();
      range.moveStart("character", start);
      range.moveEnd("character", this.elm.value.length + 1);
      range.select();
    } else if (this.elm.setSelectionRange) {
      this.elm.setSelectionRange(start, this.elm.value.length);
    }
  };

  /** @private */
  this.select = function(pos) {

    this.unselect();

    if (pos < 0) this.pos = 0;
    else if (this.pos >= this.len) this.pos = this.len - 1;
    else this.pos = pos;

    var row = document.getElementById("_sx_tr" + pos);
    if (row) {
      this.setStyles(row,this.mergeStyles(this.props.rowStyle,this.props.rowHilightStyle),true);
    }

    this._setValue();

    if (this.props.onFocusOptionHandler && this.notifiedFocusOptionPos != this.pos) {
      this.notifiedFocusOptionPos = this.pos;
      this.runHandler("onFocusOptionHandler");
    }

    if (this.boolProp("autoComplete")) {
      var v = this.getFieldValue(this.props.suggestField,pos);
      if (v) {
        this.elm.value = v;
        if (this.hasFocus) this.selectRange(this.cpos);
      }
    }


  };

  /** @private */
  this.unselect = function() {
    var row = document.getElementById("_sx_tr" + this.pos);
    if (row) this.setStyles(row,this.mergeStyles(this.props.rowStyle,this.props.rowHilightStyle),false);
  };

  /** @private */
  this.getStyles = function (styles,alt) {
    if (!styles) return "";
    if (styles && styles.startsWith(".")) {
      return this.getCss(styles,alt);
    }
    var result = "";
    var ss = styles.split(';');
    for (var i = 0; i < ss.length; i++) {
      var x = ss[i].trim();
      if (x) {
        var sn = x.before(":").trim();
        var sv = x.after(":").trim();
        sv = alt == true ? sv.after("/",sv) : sv.before("/",sv);
        if (result) result += ";";
        result += sn + ":" + sv;
      }
    }
    return result;
  };

  /** @private */
  this.getCss = function(css,alt) {
    if (alt == true) {
      return css.after("/","");
    } else {
      return css.before("/",css);
    }
  };

  /** @private */
  this.setStyles = function (row,styles,alt) {
    if (!styles) return null;
    if (styles && styles.startsWith(".")) {
      return this.setCss(row,styles,alt);
    }
    var ss = styles.split(';');
    for (var i = 0; i < ss.length; i++) {
      var x = ss[i].trim();
      if (x) {
        var sn = x.before(":").trim();
        var sv = x.after(":").trim();
        var p = sn.indexOf('-');
        if (p > -1) {
          sn = sn.substring(0,p) + sn.charAt(p+1).toUpperCase() + sn.substring(p+2);
        }
        sv = alt ? sv.after("/") : sv.before("/");
        if (sn.length > 0) row.style[sn] = sv;
      }
    }
    return null;
  };

  /** @private */
  this.mergeStyles = function(styles1,styles2) {
    if (styles1 && styles1.startsWith(".")) return styles1;
    var arr = new Array();
    this.processMerge(arr,styles1);
    this.processMerge(arr,styles2,true);

    var result = "";
    for (var sn in arr) {
      var so = arr[sn];
      if (result) result += ";";
      result += sn + ":";
      if (so.normal) result += so.normal;
      if (so.alt) result += "/" + so.alt;
    }
    return result;
  };

  /** @private */
  this.processMerge = function(arr,styles,hi) {
    if (!styles) return;
    styles = styles.split(";");
    for (var i = 0; i < styles.length; i++) {
      var style = styles[i].trim();
      if (style) {
        var sn = style.before(":");
        var sv = style.after(":");
        if (sn && sv) {
          var so = arr[sn];
          if (!so) {
            so = new Object();
            arr[sn] = so;
          }
          if (!hi) {
            so.normal = sv.before("/",sv);
            if (sv.after("/")) so.alt = sv.after("/");
          } else {
            if (sv.after("/")) so.normal = sv.after("/");
            so.alt = sv.before("/",sv);
          }
        }
      }
    }
  };

  /** @private */
  this.setCss = function(node,css,alt) {
    if (css.indexOf("/") == -1) return;
    if (alt) {
      css1 = css.before("/").substring(1);
      css2 = css.after("/").substring(1);
    } else {
      css2 = css.before("/").substring(1);
      css1 = css.after("/").substring(1);
    }
    var result = "";
    if (css1) {
      var names = node.className.split(" ");
      for (var i = 0; i < names.length; i++) {
        if (names[i] != css1) {
          if (result) result += " ";
          result += names[i];
        }
      }
    } else result = node.className;
    if (css2) result += " " + css2;
    node.className = result;
  };

  /** @private */
  this.matchSuggested = function(resp) {

    for (var i = 0; i < resp.json.item.length; i++) {
      try {
        var v = this.resp.get(i).get(this.props.suggestField);
        if (v == this.elm.value) {
          this.pos = i;
          return true;
          // v = this.resp.get(i).get(this.valueField);
        }
      } catch(e) {}
    }
    return false;

  };

  /** @private */
  this.getVal = function(arr,x,def) {
    if (!arr || !arr[x]) {
      return def;
    }
    return arr[x].trim();
  };
  
  /** @private */
  this.getAVal = function(attr,arr,x,def) {
    if (!arr || !arr[x]) {
      return def;
    }
    return ' ' + attr + '="' + arr[x] + '"';
  };
  
  

  /** @private */
  this.buildDropDownBox = function(resp) {

    if (this.sendPending) return;

    this.resp = resp;

    this.notifiedFocusOptionPos = null;

    if (this.props.onReceiveResponseHandler) {
      if (this.runHandler("onReceiveResponseHandler") == false) return;
    }

    this.len = resp.json.item.length;

    if (this.hasFocus == false) {
      if (this.boolProp("autoComplete")) {
        this.pos = 0;
        this._setValue();
      } else if (this.matchSuggested(resp)) {
        this._setValue(true);
      }
      return;
    }

    if (this.boolProp("autoComplete")) {
      this.pos = 0;
      this._setValue();
    }

    this.pos = -1;

    if (this.sendPending) return;

    var div = this.getDiv();

    var x = findPosX(this.elm) * 1;
    var offs = 1;
    if (this.props.align) {
      if (this.props.align == "right") {
        offs = this.numberProp("elmWidth",300) - this.numberProp("width",300) + 1;
      } else if (this.props.align == "center") {
        offs = ((this.numberProp("width",300) - this.numberProp("elmWidth",300)) / 2) * -1;
      }
    }

    x = x + offs;

    var y = findPosY(this.elm) * 1;
    y = y + this.numberProp("height",22);

    var fixed_style = "position:absolute; z-index : 100; top:" + y + "px; left:" + x + "px; width:" + this.props.width + "px;";

    this.pi = resp.getPagingInfo(this.props.dataSource);

    var colFields = (this.props.colFields || "").split(',');
    var colWidths = (this.props.colWidths || "").split(',');
    var colAligns = (this.props.colAligns || "").split(",");
    
    for (var i = 0; i < colWidths.length; i++) {
      var s = colWidths[i];
      if (s && !isNaN(s.substring(s.length-1))) {
        colWidths[i] = s + "%";
      }      
    }
      
    var table = '<table width="' + this.props.width + '" cellspacing="0" cellpadding="2"' + this.styleOrCss(this.props.boxStyle) + '>';
    
    if (this.props.hdrCaptions) {
      var hdr = this.props.hdrStyle ? '<tr' + this.styleOrCss(this.replaceSymbols(this.props.hdrStyle)) + '>' : '<tr>'; 
      var caps = this.props.hdrCaptions.split(",");
      var hstyles = (this.props.hdrStyles || "").split("|");
      
      for (var j = 0; j < caps.length; j++) {

        hdr += '<td' + this.getAVal("align",colAligns, j,"left") + this.styleOrCss(this.replaceSymbols(this.getVal(hstyles,j,"")));
        if (i == 0) table += this.getAVal("width",colWidths,j,"");
        hdr += '>' + caps[j] + '</td>';
      }
      hdr += '</tr>';      
      
      table += hdr;
    }

    this._inTableBuild = true;

    for (i = 0; i < resp.json.item.length; i++) {

      this.rowIndex = i;
      this.runHandler("onBeforeRowPaintHandler");

      var sbox_enter = "SBoxMgr.ENTER('" + this.elm.id + "')";
      var sbox_select = "SBoxMgr.SELECT(" + i + ", '" + this.elm.id + "')";

      table += '<tr id="_sx_tr' + i + '"' + this.styleOrCss(this.replaceSymbols(this.getStyles(this.mergeStyles(this.props.rowStyle,this.props.rowHilightStyle),false))) +
               ' onkeydown="return false;" onmouseover="' + sbox_select + '" onmousedown="' + sbox_enter + '">';

      s = "| | | | | |";
      if (this.props.colStyles) s = this.props.colStyles + s;
      var colStyles = s.split("|");

      for (j = 0; j < colFields.length; j++) {

        var f = colFields[j].trim();
        var val = f ? this.getSymbolValue(f) : "";

        table += '<td' + this.getAVal("align",colAligns, j, "left") + this.styleOrCss(this.replaceSymbols(this.getVal(colStyles,j,"")));
        if (i == 0) table += this.getAVal("width",colWidths,j,"");
        table += '>' + val + '</td>';
      }
      table += '</tr>';
    }
    table += '</table>';

    this._inTableBuild = false;

    if (this.sendPending) return;

    var sbar = "";
    var pgnr = "";
    if (this.pi.morePages || this.pi.currentPage > 1) {
      pgnr = OMgr.resource("SBOX_PAGE",this.pi.currentPage + (this.pi.morePages ? ' ... ' : '') );
    }

    // if (pgnr)
    {
      sbar += '</table>';
      var sbstyle = this.styleOrCss(this.getStyles(this.replaceSymbols(this.props.sbarStyle),!this.pi.morePages));
      var hint = ' title="' + OMgr.resource("SBOX_HINT") + '"';
      sbar += '<table width="100%" cellspacing="0" cellpadding="2"' + sbstyle + hint + '>';
      sbar += '<tr>'
      sbar += '<td>' + pgnr + ' </td>';

      if (this.pi.pagesPerBlock > 0 || this.pi.lastPage) {
        noi = OMgr.resource("SBOX_NUMBER_OF_ITEMS",this.pi.numberOfItems);
      } else {
        noi = "";
      }

      sbar += '<td></td>';
      sbar += '<td align="right">' + noi + '</td>';
      sbar += '</tr>';
    }

    if (this.sendPending) return;

    div.innerHTML = '<div style="' + fixed_style + '">' + table + sbar + '</div>';


    HideDropDowns(div, x, y);

    if (this.navigateUp) {
      this.bottom();
      this.navigateUp = false;
    } else {
      if (this.len > 0 && this.boolProp("autoComplete")) this.down();
    }

  };

  /** @private */
  this.styleOrCss = function(style) {
    if (!style) return "";
    if (style.startsWith(".")) {
      return ' class="' + style.substring(1) + '"';
    } else {
      return ' style="' + style + '"';
    }
  };

  /**
   * This method is meant to be used during building the rows of the DropDownBox or while
   * navigating over the options of the DropDownBox. It replaces the $() symbols in the given string
   * with either the symbol values from symbol pool or the values of the row columns.
   * @param s The string that contains $() symbols.
   * @return The resolved string
   * @type String
   */
  this.replaceSymbols = function(s) {
    if (!s) return "";
    var sp = s.indexOf('$(');
    if (sp == -1) return s;

    var found = false;
    var ep = 0;
    for (ep = sp + 2; ep < s.length; ep++) {
      if (s.charAt(ep) == ')') {
        found = true;
        break;
      }
    }

    if (found) {
      var sym = s.substring(sp+2,ep);
      var val = this.getSymbolValue(sym);
      s = s.substring(0,sp) + val + s.substring(ep+1);
    }

    return this.replaceSymbols(s);
  };

  /** @private */
  this.symbols = null;

  /**
   * Returns the value of the given symbol from the symbol pool
   * @symbol The name of the symbol
   * @return The value of the symbol
   * @type Syting
   */
  this.getSymbolValue = function(symbol) {
    var result = this.symbols != null ? this.symbols[symbol] : null;
    if (result) return result;
    return this.getFieldValue(symbol);
  };

  /**
   * Sets the value of a pool symbol. If the symbol does not exist in the pool
   * it is created. This function is typically used in a onBeforeRowPaint event handler
   * defined in <b>props.onBeforeRowPaint</b> property to manipulate the content of the DropDownBox row
   * being painted.
   * @param symbol The name of the symbol
   * @param value The value of the symbol
   */
  this.setSymbolValue = function(symbol, value) {
    if (this.symbols == null) this.symbols = new Array();
    this.symbols[symbol] = value;
  };

  /**
   * Returns the value of the field from specified row.
   * @param fieldName The name of the field
   * @param index Row Index (If omitted current row is used)
   * @return The value of the field
   * @type String
   */
  this.getFieldValue = function(fieldName, index) {
    if (index == null) index = this._inTableBuild ? this.rowIndex : this.pos;
    try {
      return this.resp.get(index).get(fieldName);
    } catch(e) {}
    return "";
  };


  /** @private */
  this.ajaxRefreshHandler = function(rec,nodeNames) {

    var sugField = null;
    var valField = null;
    if (nodeNames) {
      sugField = nodeNames.before("/",nodeNames);
      valField = nodeNames.after("/",null);
    }
    if (!sugField) sugField = this.props.suggestField;
    if (!valField) valField = this.props.valueField;

    if (rec) {
      if (!rec.owner.hasField(sugField)) {
        throw new Exception("RESPONSE_FIELD_MISSING",sugField);
      }
      if (!rec.owner.hasField(valField)) {
        throw new Exception("RESPONSE_FIELD_MISSING",valField);
      }

      this.elm.value  = rec.get(sugField) || "";
      this.selm.value = rec.get(valField) || "";

    } else {
      this.elm.value = "";
      this.selm.value = "";
    }

  };

  this.setValue = function(value) {
    this.selm.value = value.before(";",value,"");
    this.elm.value = value.after(";",value,"");
  };

  this.getValue = function() {
    return this.selm.value + ";" + this.elm.value;
  };

  /** Validates this SuggestBox field and returns <code>true</code> if validation is successful,
   * <code>false</code> otherwise.
   * @type boolean
   */
  this.validate = function() {
    if (this._validate() == false) return false;
    if (this.valHandler() && this.runValidationHandler() == false) return false;
    return true;
  };

  this.getProcessFieldNames = function(seperator) {
    return this.name + seperator + this.props.valueFieldName;
  };

  this.appendRequestParams = function(acc) {
    acc.add(this.elm.name,this.elm.value);
    acc.add(this.selm.name,this.selm.value);
  };

  this.setSubmitPrefix = function(prefix) {
   this.elm.name = prefix + this.name;
   this.selm.name = prefix + this.props.valueFieldName;
  };


  /** @private */
  this._validate = function() {

    var req = (this.props.required).charAt(0);

    if (req == "2" || req == "3" || req == "5") {
      if (this.elm.value == "") {
        if (this.notify(this.elm,"CANNOT_BE_BLANK") == false) return false;
      }
    }

    if (req == "3" || req == "4") {
      if (this.elm.value != "" && this.selm.value == "") {
        if (this.notify(this.elm,"NO_ASSOCIATED_VALUE") == false) return false;
      }
    }
    if (req == "5") {
      if (this.selm.value != "") {
        if (this.notify(this.elm,"ASSOCIATED_VALUE_NOT_ALLOWED") == false) return false;
      }
    }

    return true;
  };

  /**
   * Cancels all the previous requests that are in response pending status. Please note that
   * this method cannot cancel the requests that were sent with {@link #waitForResponse} option.
   */
  this.cancelRequest = function() {
    AjaxMgr.cancel(this.elm.id);
  };

}


/**
 * @class
 * Manager Class that provides static methods for SuggestBox functionalities. A static object of this class
 * is avaialable with name <CODE>SBoxMgr</CODE>.
 * <PRE>var SBoxMgr = new SuggestBoxManager();</PRE>
 */
function SuggestBoxManager() {

  /** @private */
  this.boxes = new Array();

  /**
   * Initializes and returns the {@link SuggestBox} object associated with the given DOM Node.
   */
  this.getSuggestBox = function(node) {
    if (!node.id) node.id = OMgr.getUniqueId();
    var box = this.boxes[node.id];
    if (!box) {
      box = OMgr.getObject(node);
      this.boxes[node.id] = box;
    }
    return box;
  };

  /**
   * Informs the specified SuggestBox Field object that its <code>TEXT INPUT</code> element has got the focus.
   * @param node The DOM Node of the <code>TEXT INPUT</code> element
   * @type boolean
   */
  this.onFocus = function(node) {
    return this.getSuggestBox(node).focus();
  };


  /**
   * Informs the specified SuggestBox Field object that its <code>TEXT INPUT</code> element has lost the focus.
   * @param node The DOM Node of the <code>TEXT INPUT</code> element
   * @type boolean
   */
  this.onBlur = function(node) {
    return this.getSuggestBox(node).blur();
  };

  /**
   * Informs the specified SuggestBox Field object that its <code>TEXT INPUT</code> element has fired an "onkeydown" event.
   * @param e The event object
   * @param node The DOM Node of the <code>TEXT INPUT</code> element
   * @type boolean
   */
  this.onKeyDown = function(e,node) {
    return this.getSuggestBox(node).onKeyDown(e);
  };

  /**
   * Informs the specified SuggestBox Field object that its <code>TEXT INPUT</code> element has fired an "onkeyup" event.
   * @param e The event object
   * @param node The DOM Node of the <code>TEXT INPUT</code> element
   * @param params The request parameters. It can be a single String
   * in <CODE>'p1=p1val&amp;p2=p2val'</CODE> form or a properties object in <CODE>{p1:'p1val',p2:this.value}</CODE> form
   * @type boolean
   */
  this.onKeyUp = function(e,node,params) {
    return this.getSuggestBox(node).onKeyUp(e,params);
  };


  /** @private */
  this.sendRequest = function(elmid, params) { 

    var sbi = elmid.isSuggestBox ? elmid : this.getSuggestBox(document.getElementById(elmid));

    if (sbi.sendPending) {
      var now = (new Date()).getTime();
      var diff = now - sbi.time;
      if (diff < sbi.numberProp("delay",500)) {
        setTimeout('SBoxMgr.sendRequest("' + elmid + '")',diff);
        return;
      }
    }
    
    if (sbi.props.onBeforeSendRequestHandler) {
      if (sbi.runHandler("onBeforeSendRequestHandler") == false) {
        sbi.sendPending = false;
        sbi.closeDropDownBox();
        return;
      }
    }
    
    if (params) {
      sbi.requestParams = null;  
      sbi._params = sbi.params;
      sbi.params = params;
    }

    sbi.sendPending = false;

    sbi.vlen = sbi.elm.value.length;
    sbi.prev_value = sbi.elm.value;

    if (!sbi.requestParams) {
      sbi.requestParams = new RequestParameterAccumulator(sbi.params);
      sbi.requestParams.add("_cfg",sbi.props.actionName);
    }

    var ar = new AjaxRequest(sbi.requestParams, null, sbi);
    ar.requestParams.add("_refreshds","ds:" + sbi.props.dataSource + ";cp:1");

    ar.pageInfo = sbi.props.pageInfo || OMgr.getPageInfo(sbi.node);
    ar.waitForResponse = sbi.boolProp("waitForResponse");

    ar.responseHandler = "SBoxMgr.buildDropDownBox";

    ar.send(elmid);
  };

  this.navigateWithMouse = function(elmid, page) {
    SBoxMgr.navigateTo(elmid, page);
    return false;
  };

  this.navigateTo = function(elmid, page) {

    var node = document.getElementById(elmid);
    var sbi = this.getSuggestBox(node);

    var cp = sbi.pi ? sbi.pi.currentPage : 0;

    if (sbi.valueCleared) {
      sbi.params = "";
      cp = 0;
    }

    if (sbi._params) sbi.params = sbi._params;

    sbi.requestParams = new RequestParameterAccumulator(sbi.params);
    sbi.requestParams.add("_cfg",sbi.props.actionName);

    if (page) {
      if (page == "nextPage") cp += 1;
      else if (page == "previousPage") cp -=  1;
      else if (page == "firstPage") cp = 1;
      else if (page == "lastPage") cp = -1;
    }

    if (sbi.props.onBeforeSendRequestHandler) {
      if (sbi.runHandler("onBeforeSendRequestHandler") == false) {
        sbi.sendPending = false;
        sbi.closeDropDownBox();
        return;
      }
    }

    sbi.sendPending = false;

    sbi.vlen = sbi.elm.value.length;
    sbi.prev_value = sbi.elm.value;

    var ar = new AjaxRequest(sbi.requestParams, null, sbi);

    ar.requestParams.add("_refreshds","ds:" + sbi.props.dataSource + ";cp:" + cp);

    ar.pageInfo = sbi.props.pageInfo || OMgr.getPageInfo(sbi.node);
    ar.waitForResponse = sbi.boolProp("waitForResponse");

    ar.responseHandler = "SBoxMgr.buildDropDownBox";

    ar.send(elmid);

 };


  /**
   * Cancels all the previous request that are in response pending status. Please note that
   * this method cannot cancel the requests that were sent with {@link #waitForResponse} option.
   * @param elmid The Id of the SuggestBox
   */
  this.cancelRequest = function(elmid) {
    AjaxRequest.cancel(elmid);
  };

  /** @private */
  this.buildDropDownBox = function(resp) {
    resp.request.props.buildDropDownBox(resp);
  };

  /** @private */
  this.ENTER = function(elmid) {
    var node = document.getElementById(elmid);
    var sbi = this.getSuggestBox(node);
    sbi.enter();
  };

  /** @private */
  this.SELECT = function(pos,elmid) {
    var node = document.getElementById(elmid);
    var sbi = this.getSuggestBox(node);
    sbi.select(pos);
  };

}

/** Global Singleton SuggestBoxManager object
 * @type SuggestBoxManager
 */
var SBoxMgr = new SuggestBoxManager();

/** @ignore */
function _ignore() {
  new SuggestBox();
}