/* @compact */

/* Copyright (C) MOR YAZILIM. http://www.moryazilim.com
 * All rights reserved. This software is published under the terms of the 
 * GNU Lesser General Public License. http://www.gnu.org/licenses/lgpl.txt
 */


/**
 * @fileoverview
 * Contains Button classes
 * @author Erkan Cengiz
 * @version $Id: Buttons.js 22 2008-03-06 15:36:13Z erkan $
 */

/**
 * @class
 * Represents an Animated Button. This class wraps an HTML <code>IMG</code> element with the
 * following attributes. Four images should be supplied for Normal, Mouse Over,
 * Mouse Down and Disabled modes of the button.
 * <PRE><CODE>
 * &lt;img
 *   src="button/images/normal.gif"
 *   mo:type="AnimatedButton" mo:needsInit="true"
 *   mo:props="
 *     normalSource   : 'button/images/normal.gif',
 *     mouseOverSource: 'button/images/mouseover.gif',
 *     mouseDownSource: 'button/images/mousedown.gif',
 *     disabledSource : 'button/images/disabled.gif'
 *     "
 * /&gt;
 * </CODE></PRE>
 *
 * @param node the DOM Element node that is wrapped by this object
 *
 */
function AnimatedButton(node) {

  /**
   * AnimatedButton Properties Object.
   *
   * <br/><br/><table border="1" style="border-collapse: collapse" cellspacing="3">
   <tr><td><b>normalSource</b></td><td> The image source for the "normal" mode of this button </td>
   <tr><td><b>mouseOverSource</b></td><td> The image source for the "mouseOver" mode of this button </td>
   <tr><td><b>mouseDownSource</b></td><td> The image source for the "mouseDown" mode of this button </td>
   <tr><td><b>disabledSource</b></td><td> The image source for the "disabled" mode of this button </td>
   <tr><td><b>isDisabled</b></td><td> Denotes that this button is not functional </td>
    </table>
   */
  this.props = null;

  this.base = MoreMotionObject;
  this.base(node);
  this.isAnimatedButton = true;

  /**
   * The HTML <code>IMG</code> element object
   * @type HTMLELement
   */
  this.img = node;


  /**
   * Sets the disabled status of this Button object.
   * @param value The boolean value, either <code>true</code> or <code>false</code>.
   */
  this.setDisabled = function(value) {
    if (value == true) {
      this.props.isDisabled = true;
      this.img.src = this.props.disabledSource;
      this.img.style.cursor = "default";
      if (this.img.onclick) {
        this.props.savedOnclick = this.img.onclick + "";
        this.img.onclick = "";
      }
    } else {
      this.props.isDisabled = false;
      this.img.src = this.props.normalSource;
      this.img.style.cursor = "pointer";
      if (this.props.savedOnclick) {
        eval("this.img.onclick = " + this.props.savedOnclick);
      }
    }
    this.saveProps();
  };


  /** @private */
  this.setEvent = function(eventName, jsCode) {

    var s = "";
    eval("var event = this.node." + eventName);
    if (event) {
      s = event + "";
      s = s.after("{",s);
      s = s.beforeLast("}",s);
      if (s.indexOf(jsCode) > -1) return;
    }

    eval ("this.node." + eventName + " = function() { " + jsCode + s + " }");
  };

  /** @private */
  this.setEvents = function() {
    this.setEvent("onmouseover","ABMgr.mouseOver(this);");
    this.setEvent("onmouseout","ABMgr.mouseOut(this);");
    this.setEvent("onmousedown","ABMgr.mouseDown(this);");
    this.setEvent("onmouseup","ABMgr.mouseUp(this);");
  };

  /** @private */
  this.setSources = function() {

    /** @private */
    this.mouseOverImage = new Image();
    /** @private */
    this.mouseOverImage.src = this.props.mouseOverSource;

    /** @private */
    this.mouseDownImage = new Image();
    /** @private */
    this.mouseDownImage.src = this.props.mouseDownSource;

    /** @private */
    this.disabledImage = new Image();
    /** @private */
    this.disabledImage.src = this.props.disabledSource;

  };

  /** @private */
  this.select = function() {
    this.isSelected = true;
    this.img.src = this.props.selectedSource;
  };

  /** @private */
  this.init = function() {

    this.setEvents();
    this.setSources();

    this.setDisabled(this.boolProp("isDisabled"));

  };

};
/**
 * @class
 * Manager class that provides static methods to manage the {@link AnimatedButton} objects. A static object of this class
 * is avaialable with name <CODE>ABMgr</CODE>.
 * <PRE>var ABMgr = new AnimatedButtonManager();</PRE>
 */
function AnimatedButtonManager() {

  this.current = null;

  this.getButton = function(node) {
    if (!this.current || this.current != node) {
      this.current = OMgr.getObject(node);
    }
    return this.current;
  };

  /**
   * Informs the specified AnimatedButton about a mouseover event
   * @param node The DOM Node of the {@link AnimatedButton} object
   */
  this.mouseOver = function(node) {
    var b = this.getButton(node);
    if (!b.boolProp("isDisabled"))
      b.img.src = b.props.mouseOverSource;
  };

  /**
   * Informs the specified AnimatedButton about a mousedown event
   * @param node The DOM Node of the {@link AnimatedButton} object
   */
  this.mouseDown = function(node) {
    var b = this.getButton(node);
    if (!b.boolProp("isDisabled"))
      b.img.src = b.props.mouseDownSource;
  };

  /**
   * Informs the specified AnimatedButton about a mouseout event
   * @param node The DOM Node of the {@link AnimatedButton} object
   */
  this.mouseOut = function(node) {
    var b = this.getButton(node);
    if (!b.boolProp("isDisabled"))
      b.img.src = b.props.normalSource;
  };

  /**
   * Informs the specified AnimatedButton about a mouseup event
   * @param node The DOM Node of the {@link AnimatedButton} object
   */
  this.mouseUp = function(node) {
    var b = this.getButton(node);
    if (!b.boolProp("isDisabled"))
      b.img.src = b.props.normalSource;
  };

};

/**
 * Global Singleton AnimatedButtonManager object
 * @type AnimatedButtonManager
 */
var ABMgr = new AnimatedButtonManager();