/* @compact */

/* Copyright (C) MOR YAZILIM. http://www.moryazilim.com
 * All rights reserved. This software is published under the terms of the 
 * GNU Lesser General Public License. http://www.gnu.org/licenses/lgpl.txt
 */


/**
 * @fileoverview
 * Contains FileBox and FileBoxManager classes
 * @author Erkan Cengiz
 * @version $Id: Filer.js 136 2008-05-11 17:21:22Z erkan $
 */

 /**
  * @class FileBox
  * Represents a FileBox that wraps a <code>FILE INPUT</code> element and moderates the opening of a
  * File Upload Window to select a file to upload and uploading the selected file to the server.
  * @extends ProcessField
  * @param node the DOM Element node that is wrapped by this field
  */
 function FileBox(node) {  

  /**
   * FileBox Properties Object.
   *
   *<br/><br/>
   * <table border="1" style="border-collapse: collapse" cellspacing="3">
   * <tr><td><b>winUrl</b></td><td>The URL of the File Upload Window </td></tr>
   * <tr><td valign="top"><b>allowedExts</b></td><td>The list of the allowed file extensions (separated with ';' character) that can be
   * selected in the File Dialog where the user selects a file to upload. Example ".jpg;.gif" </td></tr>
   * <tr><td><b>winWidth</b></td><td>The width of the File Upload Window</td></tr>
   * <tr><td><b>winHeight</b></td><td>The height of the File Upload Window </td></tr>
   * <tr><td><b>winToolBars</b></td><td>The ToolBars flag for which <code>true</code> means that the toolbars should be visible on the File Open Dialog.</td></tr>
   * <tr><td><b>winScrolBars</b></td><td>The ScrolBars flag for which <code>true</code> means that the scrolbars should be visible on the File Open Dialog</td></tr>
   * <tr><td><b>winStatusBar</b></td><td>The StatusBar flag for which <code>true</code> means that the statusbar should be visible on the File Open Dialog.</td></tr>
   * <tr><td><b>winMenuBar</b></td><td>The MenuBar flag for which <code>true</code> means that menu bar should be visible on the File Open Dialog</td></tr>
   * <tr><td><b>winResizeable</b></td><td>The Resizable flag for which <code>true</code> means that the window of the File Open Dialog can be resized</td></tr>
   * <tr><td><b>winLocation</b></td><td>The Location flag for which <code>true</code> means that the Location input field of the browser should be visible on the File Open Dialog</td></tr>
   * <tr><td><b>onFileSelectHandler</b></td><td>The name of the event handler function that will be called when a file is selected to upload</td></tr>
   * <tr><td><b>onFileUploadHandler</b></td><td>The name of the event handler function that will be called when the uploading
   * of the selected file is completed</td></tr>
   * <tr><td><b>uploadId</b></td><td></td></tr>
   * </table>
   *
   */
  this.props = null;

  this.base = ProcessField;
  this.base(node);
  /** @type boolean */
  this.isFileBox = true;

  /**
   * The browser name of the uploaded file
   * @type String
   */
  this.browserFileName = null;
  /**
   * The server name of the uploaded file
   * @type String
   */
  this.serverFileName = null;


  /** @private */
  this.wrapElements = function() {
    var elms = this.node.getElementsByTagName("input");
    if (elms && elms.length) {
      for (var i = 0; i < elms.length; i++) {
        var elm = elms[i];
        if (elm.name.startsWith("pf_") || elm.name == this.name) {
          this.elm = elm;
          break;
        }
      }
    }
  };

  this.wrapElements();

  
  this.setValue = function(value) {
    this.elm.value = value;
  };

  this.getValue = function() {
    return this.elm.value;
  };
  
  this.clear = function() {
    this.elm.value = "";
    this.browserFileName = null;
    this.serverFileName = null;  
  };  

  this.validate = function() {
    var elm = this.elm;
    if (this.boolProp("nonBlank") && this.checkValueIsAvailable(elm) == false) return false;
    if (this.valHandler() && this.runValidationHandler() == false) return false;
    return true;
  };

  /**
   * Opens the file dialog where the user can select a file from the local computer and upload to the server. The
   * name of the uploaded file is returned to the value property of the <code>FILE INPUT</code> element that is
   * wrapped by this object.
   */
  this.openFileDialog = function() {

    var url = this.props.winUrl;
    if (!url) url = "/moremotion/Filer/FileUploadWindow";

    var ua = new UrlAddress(url);

    if (!this.node.id) this.node.id = OMgr.getUniqueId();
    ua.addParam("elmid",this.node.id);

    WinMgr.openWindow( {
      url        : ua,
      left       : this.winLeft,
      top        : this.winTop,
      width      : this.props.winWidth,
      height     : this.props.winHeight,
      toolbar    : this.boolProp("winToolBars"),
      scrollbars : this.boolProp("winScrollBars"),
      status     : this.boolProp("winStatusBar"),
      menubar    : this.boolProp("winMenuBar"),
      resizable  : this.boolProp("winResizeable"),
      location   : this.props.winLocation
      } );

  };

}

/**
 * @class
 * Manager Class that provides static methods for FileBox functionalities. A static object of this class
 * is avaialable with name <CODE>FileBoxMgr</CODE>.
 * <PRE>var FileBoxMgr = new FileBoxManager();</PRE>
 */
function FileBoxManager() {

  /**
   * Opens the file dialog where the user can select a file from the local computer and upload to the server. The
   * name of the uploaded file is returned to the value property of the specified <code>FILE INPUT</code> element.
   * @param node The DOM node which is enclosed by the FileBox
   */
  this.openFileDialog = function(node) {
    var f = OMgr.getEnclosingObject(node);
    if (f) f.openFileDialog();
  };

  /** @private */
  this.validateExtension = function(filebox, frm) {

    var allowedExts = filebox.dynProp("allowedExts");
    if (!allowedExts) return true;

    var ffn = frm.bfn.value;
    var exts = allowedExts.split(';');
    for (var i = 0; i < exts.length; i++) {
      var ext = exts[i].trim().toLowerCase();
      if (ext.startsWith("*")) ext = ext.after("*");
      if (ffn.toLowerCase().endsWith(ext)) {
        return true;
      }
    }

    return false;

  };

  /** @private */
  this.submitFile = function(fileboxId, frm) {

    var node = window.opener.document.getElementById(fileboxId);
    var filebox = window.opener.OMgr.getObject(node);

    if (frm.bfn.value == "") {
      alert( OMgr.resource("CANNOT_BE_BLANK") );
      frm.bfn.focus();
      return false;
    }

    if (this.validateExtension(filebox, frm) == false) {
      alert( OMgr.resource("EXTENSION_NOT_ALLOWED") );
      frm.bfn.focus();
      return false;
    }

    filebox.browserFileName = frm.bfn.value;

    if (filebox.props.onFileSelectHandler)
      if (filebox.runHandler("onFileSelectHandler") == false) return false;

    frm.fbi.value = fileboxId;
    frm.uli.value = filebox.props.uploadId;
    frm.lfn.value = frm.bfn.value;

    frm.submit();
    
    return true;
  };

  /** @private */
  this.uploadSuccess = function(fileboxId, serverFileName) {

    var node = window.opener.document.getElementById(fileboxId);
    var filebox = window.opener.OMgr.getObject(node);
    filebox.serverFileName = serverFileName;
    filebox.elm.value = serverFileName;

    if (filebox.props.onFileUploadHandler) {
      if (filebox.runHandler("onFileUploadHandler") == false) return false;
    }
    filebox.setModified();
    
    return true;
  };

}
var FileBoxMgr = new FileBoxManager();

/** @ignore */
function _ignore() {
  new FileBox();
}