/* @compact */

/* Copyright (C) MOR YAZILIM. http://www.moryazilim.com
 * All rights reserved. This software is published under the terms of the 
 * GNU Lesser General Public License. http://www.gnu.org/licenses/lgpl.txt
 */


/**
 * @fileoverview
 * Contains classes to manage MoreMotion Tabs.
 * @author Erkan Cengiz
 * @version $Id: Tabs.js 22 2008-03-06 15:36:13Z erkan $
 */

/**
 * @class
 * Represents a Tab Control. A Tab Control contains the same amount of {@link TabButton} and {@link TabPanel} objects
 * with identical names. For Example: If a TabControl contains two tabs then there must be two TabPanels with names
 * i.e. "Tab1" and "Tab2" in a TabPanelContainer and two TabButtons with names i.e. "Tab1" and "Tab2" in a TabButtonContainer.
 * <p>
 * Below is an example HTML code that contains necessary definitions for a TabControl. It is not
 * important what type of HTML elements are used as containers. i.e."TABLE" elements could also be used
 * instead of "DIV" elements. The only requirement is to use the "mo:*" attributes correctly.
 * <p>
 *
 * <pre>
 * &lt;div mo:type="TabControl" mo:name="TabControl1" mo:needsInit="true" mo:props="initialTab:'Tab1'"&gt;
 *   &lt;div mo:type="TabButtonContainer"&gt;
 *     &lt;img mo:name="Tab1" src="n.gif" mo:type="TabButtonAnimated" mo:needInit="true"
 *          mo:props="normalSource:'n.gif', mouseOverSource:'mo.gif', mouseDownSource:'md.gif', selectedSource:'sel.gif'"/&gt;
 *     &lt;img mo:name="Tab2" src="n.gif" mo:type="TabButtonAnimated" mo:needInit="true"
 *          mo:props="normalSource:'n.gif', mouseOverSource:'mo.gif', mouseDownSource:'md.gif', selectedSource:'sel.gif'"/&gt;
 *   &lt;/div&gt;
 *   &lt;div mo:type="TabPanelContainer"&gt;
 *     &lt;div mo:type="TabPanel" mo:name="Tab1"&gt;Tab Area 1&lt;/div&gt;
 *     &lt;div mo:type="TabPanel" mo:name="Tab2"&gt;Tab Area 2&lt;/div&gt;
 *   &lt;/div&gt;
 * &lt;/div&gt;
 * </pre>
 *
 *
 * @param node the DOM Element node that is marked as Tab Control by means of "mo:type" attribute with
 * a value of "TabControl".
 *
 *
 */
function TabControl(node) {

  /**
   * TabControl Properties Object.
   *
   * <br/><br/><table border="1" style="border-collapse: collapse" cellspacing="3">
   * <tr><td><b>initialTab</b></td><td width="654">The name of the initially visible tab.</td>
   * <tr><td><b>onTabSelectHandler</b></td> <td width="654">The name of the onTabSelectHandler Function. The function defined here is called
   * each time a tab is selected and the handle of this object is passed to it as the only parameter</td></tr>
   * </table>
   */
  this.props = null;

  this.base = MoreMotionObject;
  this.base(node);
  this.isTabControl = true;

  /**
   * Returns the {@link TabPanel} object associated with the specified name
   * @param name The name of the TabPanel
   * @type TabPanel
   */
  this.getTabPanel = function(name) {
    var node = OMgr.findChildNode(this.node,null,"mo:type","TabPanel");
    if (node) {
      node = OMgr.findChildNode(this.node.parentNode,null,"mo:name",name);
      if (node) return OMgr.getObject(node);
    }
    return null;
  };

  /**
   * Returns an array of contained TabPanels
   * @type TabPanel[]
   */
  this.getTabPanels = function() {
    var panels = new Array();

    var node = OMgr.findChildNode(this.node,null,"mo:type","TabPanelContainer",4);
    if (node) this._collectPanels(panels, node);
    else {
      node = OMgr.findChildNode(this.node,null,"mo:type","TabPanel");
      if (node) this._collectPanels(panels, node.parentNode.parentNode);
    }
    return panels;
  };

  /** @private */
  this._collectPanels = function(panels, parentNode) {

    for (var i = 0; i < parentNode.childNodes.length; i++) {
      var node = parentNode.childNodes[i];
      if (node.nodeType == 1) {
        var attr = OMgr.getAttribute(node,"mo:type");
        if (attr) {
          if (attr == "TabPanel") {
            panels[panels.length] = OMgr.getObject(node);
          }
        } else if (node.hasChildNodes) {
          this._collectPanels(panels, node);
        }
      }
    }

  };

  /**
   * Returns an array of contained TabButtons
   * @type TabPanel[]
   */
  this.getTabButtons = function() {
    var buttons = new Array();
    var node = OMgr.findChildNode(this.node,null,"mo:type","TabButtonContainer");
    if (node) {
      this._collectButtons(buttons, node);
    }
    return buttons;
  };

  /** @private */
  this._collectButtons = function(buttons, parentNode) {

    for (var i = 0; i < parentNode.childNodes.length; i++) {
      var node = parentNode.childNodes[i];
      if (node.nodeType == 1) {
        var attr = OMgr.getAttribute(node,"mo:type");
        if (attr) {
          if (attr.startsWith("TabButton")) {
            buttons[buttons.length] = OMgr.getObject(node);
          }
        } else if (node.hasChildNodes) {
          this._collectButtons(buttons, node);
        }
      }
    }

  };

  /**
   * Returns the name of the tab that is in "selected" status.
   * @return String
   */
  this.getSelectedTabName = function() {
    var buttons = this.getTabButtons();
    for (var i = 0; i < buttons.length; i++) {
      var button = buttons[i];
      if (button.boolProp("isSelected")) {
        return button.name;
      }
    }
    return null;

  };

  /**
   * Returns the TabControl object that encloses this TabControl
   * @type TabControl
   */
  this.getParentTabControl = function() {
    var node = OMgr.findParentNode(this.node,"mo:type","TabControl");
    if (node) {
      return OMgr.getObject(node);
    }
    return null;
  };

  /**
   * Returns the TabButton object associated with the specified name
   * @param name The name of the TabButton
   * @type TabButton
   */
  this.getTabButton = function(name) {
    var node = OMgr.findChildNode(this.node,null,"mo:type","TabButtonContainer");
    if (node) {
      node = OMgr.findChildNode(this.node,null,"mo:name",name);
      if (node) return OMgr.getObject(node);
    }
    return null;
  };

  /**
   * Makes the specified {@link TabPanel} visible and others invisible.
   * @param tabName The name of the {@link TabPanel} to make visible.
   * @param recursive Flag that denotes that recursively all the enclosing TabPanels becomes visible
   */
  this.switchTo = function(tabName, recursive) {

    if (recursive) {
      var node = OMgr.findParentNode(this.node,"mo:type","TabPanel");
      if (node) {
        var panel = OMgr.getObject(node);
        node = OMgr.findParentNode(node,"mo:type","TabControl");
        OMgr.getObject(node).switchTo(panel.name,true);
      }
    }

    var panels = this.getTabPanels();

    for (var i = 0; i < panels.length; i++) {
      var panel = panels[i];
      var div = panel.node.nodeName.toUpperCase() == "DIV" ? panel.node : panel.node.parentNode;
      if (panel.name == tabName) {
        div.style.display = "block";
      } else {
        div.style.display = "none";
      }
    }

    var buttons = this.getTabButtons();
    for (var i = 0; i < buttons.length; i++) {
      var button = buttons[i];
      if (button.name == tabName) {
        button.select();
      } else {
        button.deselect();
      }
    }

    if (this.props && this.props.onTabSelectHandler) {
      var handler = new EventHandler(this.props.onTabSelectHandler, "onTabSelectHandler");
      return handler.run(this);
    }

  };

  /** @private */
  this.setEvent = function(button, eventName, jsCode) {

    var s = "";
    eval("var event = button.node." + eventName);
    if (event) {
      s = event + "";
      s = s.after("{",s);
      s = s.beforeLast("}",s);
      if (s.indexOf(jsCode) > -1) return;
    }

    eval ("button.node." + eventName + " = function() { " + jsCode + s + " }");
  };

  /** @private */
  this.setEvents = function() {
    var buttons = this.getTabButtons();
    for (var i = 0; i < buttons.length; i++) {
      var b = buttons[i];
      this.setEvent(b,"onmouseover","TabMgr.mouseOver(this);");
      this.setEvent(b,"onmouseout","TabMgr.mouseOut(this);");
      this.setEvent(b,"onmousedown","TabMgr.mouseDown(this);");
      this.setEvent(b,"onmouseup","TabMgr.mouseUp(this);");
    }
  };

  /**
   * Initializes this TabControl. Makes the initial tab visible.
   */
  this.init = function() {
    this.setEvents();
    this.switchTo(this.props.initialTab);
  };

};


/**
 * @class
 * Represents a Tab Panel that exist is a {@link TabControl}
 * @param node the DOM Element node that is marked as Tab Panel by means of "mo:type" attribute with
 * a value of "TabPanel".
 */
function TabPanel(node) {
  this.base = MoreMotionObject;
  this.base(node);
  this.isTabPanel = true;
};

/**
 * @class
 * Base class for Tab Button classes. A Tab Button class should extend this class and override its
 * abstract methods.
 * @param node the DOM Element node that is marked as Tab Button by means of setting its "mo:type" attribute to
 * a value that has the prefix "TabButton". This means the name of the class that extends this class must start "TabButton" i.e.
 * "TabButtonAnimated".
 *
 */
function TabButton(node) {

  this.base = MoreMotionObject;
  this.base(node);
  this.isTabButton = true;

  /** Abstract method that is called when the button element is clicked. */
  this.select = function() {};

  /** Abstract method that is called when the button looses the selection.  */
  this.deselect = function() {};

  /** Abstract method that is called to set a caption to the button. */
  this.setCaption = function(caption) {};

  /** Abstract method that is called when the mouse moves over to the button element. */
  this.mouseOver = function() {};

  /** Abstract method that is called when the mouse moves out from the button element. */
  this.mouseOut = function() {};

  /** Abstract method that is called when the mouse button is pressed down on the button element. */
  this.mouseDown = function() {};

  /** Abstract method. It is called when the mouse button is released on the button element. */
  this.mouseUp = function() {};

  /**
   * Returns the TabControl that this button resides in
   * @type TabControl
   */
  this.getTabControl = function() {
    if (!this._tabControl) {
      var node = OMgr.findParentNode(this.node,"mo:type","TabControl");
      if (node) {
        this._tabControl = OMgr.getObject(node);
      }
    }
    return this._tabControl;
  };

};


/**
 * @class
 * Represents a TabButtonFlex. A FlexTabutton is built out of pure text and therefore suitable for
 * Multilingual applications. The outlook of this button can be customized using CSS Styles.
 * @extends TabButton
 * @param node the DOM Element node that is marked as a Tab Button by means of "mo:type" attribute with
 * a value of "TabButtonFlex".
 */
function TabButtonFlex(node) {

  this.base = TabButton;
  this.base(node);
  this.isTabButtonFlex = true;

  /** @private */
  this.setClass = function(classSuffix) {

    var classPrefix = this.getTabControl().props.classPrefix;

    if (classSuffix == "mt") {
      classSuffix = this.boolProp("isSelected") ? "s" : "n";
    }
    var buf = "";
    var classNames = this.node.className.split(' ');
    for (var i = 0; i < classNames.length; i++) {
      var cls = classNames[i];

      if (!cls.startsWith(classPrefix) ) {
        if (buf == "") buf = cls; else buf += " " + cls;
      }
    }

    this.node.className = buf + " " + classPrefix + "_" + classSuffix;
  };

  /** Sets the "mouseover style class" to this button */
  this.mouseOver = function() {
    if (!this.boolProp("isDisabled") && !this.boolProp("isSelected")) {
      this.setClass("mo");
    }
  };

  /** Sets the "mousedown style class" to this button */
  this.mouseDown = function() {
    if (!this.boolProp("isDisabled") && !this.boolProp("isSelected")) {
      this.setClass("md");
    }
  };

  /** Sets the "normal style class" to this button */
  this.mouseOut = function() {
    if (!this.boolProp("isDisabled") && !this.boolProp("isSelected")) {
      this.setClass("mt");
    }
  };

  /** Swithes to the associated tab related to this button */
  this.mouseUp = function() {
    if (!this.boolProp("isDisabled") && !this.boolProp("isSelected")) {
      this.getTabControl().switchTo(this.name);
    }
  };

  /** Marks this tab as the selected tab and sets the "selected style class" to it */
  this.select = function() {
    this.setClass("s");
    this.props.isSelected = true;
    this.saveProps();
  };

  /** Removes the "selected tab" mark from this button and sets the "normal style class" to it */
  this.deselect = function() {
    if(!this.boolProp("isDisabled")) this.setClass("n");
    this.props.isSelected = false;
    this.saveProps();
  };

  /** Initializes this button */
  this.init = function() {
    this.node.style.cursor = "pointer";
    if(!this.boolProp("isDisabled")) this.setClass("n");
    this.setDisabled(this.props.isDisabled);
  };

  // this.init(props);

};

/**
 * @class
 * This class wrappes the Tab Control that has the type "TabControlFlex"
 * @param node the DOM Element node that is marked as a Tab Control by means of "mo:type" attribute with
 * a value of "TabControlFlex".
 */
function TabControlFlex(node) {

  /**
   * TabControlFlex Properties Object.
   *
   * <br/><br/>
   * <table border="1" style="border-collapse: collapse" cellspacing="3">
   * <tr><td valign="top"><b>classPrefix</b></td><td>
   * The prefix of the CSS Class. Please note that a set of CSS Styles should be supplied in a CSS
   * file where there are styles that have certain suffixes and the same prefixes.
   *
   * <br/><br/><u>The CSS style classes that must be provided:</u><br/>
   *
   * <br/>&nbsp;&nbsp;<code><i>classPrefix</i>_<b>mo</b> {}</code> - Mouse Over Style of the buttons
   * <br/>&nbsp;&nbsp;<code><i>classPrefix</i>_<b>md</b> {}</code> - Mouse Down Style of the buttons
   * <br/>&nbsp;&nbsp;<code><i>classPrefix</i>_<b>n</b> {}</code> - Normal Style of the buttons
   * <br/>&nbsp;&nbsp;<code><i>classPrefix</i>_<b>s</b> {}</code> - Selected Style of the buttons
   * <br/>&nbsp;&nbsp;<code><i>classPrefix</i>_<b>ta</b> {}</code> - Style of the TabControl
   * <br/>&nbsp;&nbsp;<code><i>classPrefix</i>_<b>ba</b> {}</code> - Style of the Tab Buttons Container
   * </td></tr>
   * </table>
   *
   */
  this.props = null;

  this.base = TabControl;
  this.base(node);
  this.isTabControlFlex = true;

  this.init = function() {
    this.setEvents();
    var node = OMgr.findChildNode(this.node,null,"mo:type","TabPanel");
    if (node) {
      this.setClass(node.parentNode.parentNode,this.props.classPrefix + "_ta");
    }
    var node = OMgr.findChildNode(this.node,null,"mo:type","TabButtonContainer");
    this.setClass(node,this.props.classPrefix + "_ba");
    this.switchTo(this.props.initialTab);
  };

  this.setClass = function(node, className) {
    node.className = node.className + " " + className;
  };

};


/**
 * @class
 * Represents an Animated Tab Button.
 * This class wraps an HTML <code>IMG</code> element with the
 * following attributes. Five images should be supplied for Normal, Mouse Over,
 * Mouse Down, Selected and Disabled modes of the button.
 * @extends TabButton
 * @param node the DOM Element node that is marked as a Tab Button by means of "mo:type" attribute with
 * a value of "TabButtonAnimated".
 */
function TabButtonAnimated(node) {

  /**
   * TabButtonAnimated Properties Object.
   *
   * <br/><br/>
   * <table border="1" style="border-collapse: collapse" cellspacing="3">
   * <tr><td><b>normalSource</b></td><td> The image source for the "normal" mode of this button </td></tr>
   * <tr><td><b>mouseOverSource</b></td><td> The image source for the "mouseOver" mode of this button </td></tr>
   * <tr><td><b>mouseDownSource</b></td><td> The image source for the "mouseDown" mode of this button </td></tr>
   * <tr><td><b>selectedSource</b></td><td> The image source for the "selected" mode of this button </td></tr>
   * <tr><td><b>disabledSource</b></td><td> The image source for the "disabled" mode of this button </td></tr>
   * <tr><td><b>isDisabled</b></td><td> Flag denotes that the AnmatedTabButton associated with this button is disabled and visible. </td></tr>
   * </table>
   *
   */
  this.props = null;

  this.base = TabButton;
  this.base(node);
  this.isTabButtonAnimated = true;

  /**
   * The HTML <code>IMG</code> element object
   * @type HTMLELement
   */
  this.img = this.node;

  /**
   * Sets the disabled status of this Button object.
   * @param value The boolean value, either <code>true</code> or <code>false</code>.
   */
  this.setDisabled = function(value) {
    if (value) {
      this.props.isDisabled = true;
      this.img.src = this.props.disabledSource;
      this.img.style.cursor = "default";
      this.savedOnclick = this.img.onclick;
      this.img.onclick = "";
    } else {
      this.props.isDisabled = false;
      this.img.src = this.props.normalSource;
      this.img.style.cursor = "pointer";
      if (!this.img.onclick || this.img.onclick == "") {
        this.img.onclick = this.savedOnclick || "";
      }
    }
  };

  /** @private */
  this.setSources = function() {

    /** @private */
    this.mouseOverImage = new Image();
    /** @private */
    this.mouseOverImage.src = this.props.mouseOverSource;

    /** @private */
    this.mouseDownImage = new Image();
    /** @private */
    this.mouseDownImage.src = this.props.mouseDownSource;

    /** @private */
    this.selectedImage = new Image();
    /** @private */
    this.selectedImage.src = this.props.selectedSource;
     /** @private */
    this.disabledImage = new Image();
      /** @private */
    this.disabledImage.src = this.props.disabledSource;

  };

  /** Marks this tab as the selected tab and makes the "selected image" visible */
  this.select = function() {
    this.img.src = this.props.selectedSource;
    this.props.isSelected = true;
    this.saveProps();
  };

  /** Removes the "selected tab" image and makes the "normal image" visible */
  this.deselect = function() {
    if(!this.boolProp("isDisabled")) this.img.src = this.props.normalSource;
    this.props.isSelected = false;
    this.saveProps();
  };

  /** Makes the "mouseover image" visible */
  this.mouseOver = function() {
    if (!this.boolProp("isDisabled") && !this.boolProp("isSelected")) {
      this.img.src = this.props.mouseOverSource;
    }
  };

  /** Makes the "mousedown image" visible */
  this.mouseDown = function() {
    if (!this.boolProp("isDisabled") && !this.boolProp("isSelected")) {
      this.img.src = this.props.mouseDownSource;
    }
  };

  /** Makes the "normal image" visible */
  this.mouseOut = function() {
    if (!this.boolProp("isDisabled") && !this.boolProp("isSelected")) {
      this.img.src = this.props.normalSource;
    }
  };

  /** Swithes to the associated tab related to this button */
  this.mouseUp = function() {
    if (!this.boolProp("isDisabled") && !this.boolProp("isSelected")) {
      this.getTabControl().switchTo(this.name);
    }
  };

  /** Initializes this TabButton and loads all the used image elements */
  this.init = function() {
    this.img.style.cursor = "pointer";
    this.setSources();
    this.setDisabled(this.props.isDisabled);
  };


};

/**
 * @class
 * Manager class that provides static methods to manage the Tab objects. A static object of this class
 * is avaialable with name <CODE>TabMgr</CODE>.
 * <PRE>var TabMgr = new TabManager();</PRE>
 */
function TabManager() {

  /** @private */
  this.current = null;

  /** @private */
  this.getButton = function(node) {
    if (!this.current || this.current != node) {
      this.current = OMgr.getObject(node);
    }
    return this.current;
  };

  /** @private */
  this.mouseOver = function(node) {
    var b = this.getButton(node);
    if (b) b.mouseOver();
  };

  /** @private */
  this.mouseOut = function(node) {
    var b = this.getButton(node);
    if (b) b.mouseOut();
  };

  /** @private */
  this.mouseDown = function(node) {
    var b = this.getButton(node);
    if (b) b.mouseDown();
  };

  /** @private */
  this.mouseUp = function(node) {
    var b = this.getButton(node);
    if (b) b.mouseUp();;
  };

  /**
   * Returns the {@link TabControl} object associated with the given name.
   * @param name The name of the tab control
   * @param node The DOM node object to start the searching of the specified TabControl.
   * If ommited <CODE>document.body</CODE> is the default.
   * @return TabControl
   */
  this.getTabControl = function(name, node) {
    if (!node) node = document.body;
    var node = OMgr.findChildNode(node,"type:TabControl","mo:name",name);
    if (node) {
      return OMgr.getObject(node);
    }
    return null;
  };

  /**
   * Makes the Tab that encloses the given DOM node visible.
   * @param node The DOM node object enclosed by a Tab
   */
  this.focusEnclosingTab = function(node) {
    var node = OMgr.findParentNode(node,"mo:type","TabPanel");
    if (node) {
      var panel = OMgr.getObject(node);
      node = OMgr.findParentNode(node,"mo:type","TabControl");
      OMgr.getObject(node).switchTo(panel.name,true);
    }
  };

  /**
   * Makes the specified tab of the specified TabControl visible
   * @param tcName The name of the {@link TabControl}
   * @param tabName The name of the tab
   * @param node The DOM node object to start the searching of the specified TabControl.
   * If ommited <CODE>document.body</CODE> is the default.
   */
  this.switchTo = function(tcName,tabName,node) {
    var tc = this.getTabControl(tcName,node);
    if (tc) tc.switchTo(tabName);
  };

};

var TabMgr = new TabManager();

/** @ignore */
function _ignore() {
  new TabControl();
  new TabPanel();
  new TabButton();
  new TabButtonAnimated();
};
