/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */
package org.moremotion.adom;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.util.Date;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/** Application Data Object Model (ADOM)
 * <P/>
 * ADOMs are MoreMotion's data objects to store hiearchically structured data in
 * Request, Session or Application scopes. The responsibility of a MoreMotion programmer 
 * is to create ADOMs that contains the data required by the next MoreMotion page. The converting the ADOMs to 
 * XML strings is MoreMotion's responsibility.
 * <P>The data of an ADOM is stored as {@link ADOMNode} objects which is linked to the root node of the ADOM. Note that
 * The data of an ADOM can also be plain XML string instead of ADOMNode collections.
 * </P>
 * @author Yusuf Boyacigil
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: ADOM.java 51 2008-03-10 11:46:54Z erkan $
 */
public class ADOM extends ADOMNode implements Cloneable {

  private String xmlExplicit = null;
  private long lastTimeRefreshed = -1;
  private String resbundName = null;
  private boolean isProtected = false;
  private long sequence = 0;

  /**
   * Clones this ADOM object and returns it
   */
  public Object clone() throws CloneNotSupportedException {

    ADOM copy = (ADOM) super.clone();
    return copy;

  }

  /**
   * Create a new ADOM object.
   */
  public ADOM() {
    super("*root");
    refresh();
  }

  public String getSequence() {
    return ++sequence + "";
  }

  /*
   * Sets the resource bundle name of this ADOM. If an ADOM has a resource name, i.e. <CODE>myres</CODE>, assigned and
   * if the data stored in ADOM contains <CODE>$res()</CODE> symbols, i.e. <CODE>$res(MESSAGE_ID)</CODE>, they will be replaced with the
   * resource strings found in the resource files, i.e. <CODE>myres.res</CODE> when it is converted to the
   * XML string to prepare the XML Data of the next page.
   * @param resBundName The name of the resource bundle
   */
  public void setResBundleName(String resBundName) {
    this.resbundName = resBundName;
  }

  /**
   * Gets the resource bundle name of this ADOM
   */
  public String getResBundleName() {
    return this.resbundName;
  }

  public void setProtected(boolean value) {
    this.isProtected = value;
  }

  public boolean isProtected() {
    return this.isProtected;
  }

  public ADOMNode getRootNode() {
    return this;
  }

  /**
   * Sets the value of this ADOM with a XML string.
   * <P>If this method is used for an ADOM, the XML string given will be used as it is when converting this ADOM to XML.
   * If the ADOM has child nodes they will be ignored.</P>
   * @param value The XML string.
   */
  public void setXmlExplicit(String value) {
    refresh();
    this.xmlExplicit = value;
  }

  /**
   * Checks whether this ADOM stores its data as XML string or as ADOMNodes.
   *
   * @return <CODE>true</CODE> if the ADOM stores its data as XML string, <CODE>false</CODE> otherwise.
   */
  public boolean hasXmlExplicit() {
    return (this.xmlExplicit != null);
  }

  /**
   * Returns the XML string which was set explicitly.
   */
  public String getXmlExplicit() {
    return this.xmlExplicit;
  }

  /**
   * Checks whether this ADOM requires refreshing or not.
   *
   * @param timeoutInSecs in seconds
   * @return True if refresh is required, false otherwise.
   */
  public boolean isStale(long timeoutInSecs) {
    if (timeoutInSecs == -1) {
      return false;
    }
    return ((System.currentTimeMillis() - lastTimeRefreshed) > (timeoutInSecs * 1000));
  }

  /**
   * Returns the date when this ADOM last refreshed.
   */
  public Date lastTimeRefreshed() {
    if (this.lastTimeRefreshed == -1) {
      return null;
    }
    return new Date(this.lastTimeRefreshed);
  }

  /**
   * Clears all the child nodes.
   */
  public void clear() {
    super.clear();
    sequence = 0;
  }

  /**
   * Returns the String representation of this node.
   */
  public String toString() {
    StringBuffer buffer = new StringBuffer();
    buffer.append("[ lastTimeRefreshed: " + lastTimeRefreshed());
    buffer.append(", hasXmlExplicit: " + hasXmlExplicit());
    buffer.append(" ]");
    return buffer.toString();
  }

  private void refresh() {
    lastTimeRefreshed = System.currentTimeMillis();
  }

  /**
   * Saves the ADOM to the specified file as XML string using the encoding UTF-8.
   * @param fileName Name of the target file
   * @throws ADOMException if save operation is not successful.
   */
  public void saveToFile(String fileName) throws ADOMException {
    File f = new File(fileName);
    saveToFile(f);
  }

  /**
   * Saves the ADOM to the specified file as XML string using the encoding UTF-8.
   * @param file File object of the target file
   * @throws ADOMException if save operation is not successful.
   */
  public void saveToFile(File file) throws ADOMException {

    try {

      File dir = new File(file.getParent());
      if (!dir.exists()) {
        dir.mkdirs();
      }
      OutputStreamWriter out = new OutputStreamWriter(new FileOutputStream(file), "UTF-8");

      out.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
      if (hasXmlExplicit()) {
        out.write(xmlExplicit);
      } else {
        out.write(super.toXmlWithIndent());
      }

      out.flush();
      out.close();
    } catch (Exception e) {
      throw new ADOMException("Unable to save ADOM to file " + file.getAbsolutePath(), null, e);
    }
  }
  
  private static String getValue(Node node) {
    Node n = node.getFirstChild();
    if (n == null) {
      return "";
    }
    return n.getNodeValue();
  }

  private void convElmToNode(Element elm, ADOMNode node) {

    NamedNodeMap nnm = elm.getAttributes();
    for (int i = 0; i < nnm.getLength(); i++) {
      Node attrNode = nnm.item(i);
      node.addAttribute(attrNode.getNodeName(), attrNode.getNodeValue());
    }

    NodeList nlist = elm.getChildNodes();
    for (int i = 0; i < nlist.getLength(); i++) {
      Node n = nlist.item(i);
      if (n.getNodeType() == Node.ELEMENT_NODE) {
        Element e = (Element) n;
        ADOMNode an = new ADOMNode(e.getNodeName());
        node.addNode(an);
        if (!e.hasChildNodes()) {
          an.setValue( getValue(e) );
        }

        convElmToNode(e, an);
      }
    }

  }

  /**
   * Loads the ADOM from the specified text file that contains a valid XML document.
   * @param fileName File name to load
   * @throws ADOMException if load is not successful
   */
  public void loadFromFile(String fileName) throws ADOMException {
    loadFromFile(new File(fileName));
  }

  /**
   * Loads the ADOM from the specified text file that contains a valid XML document.
   * @param file File object of the file to load
   * @throws ADOMException if load is not successful
   */
  public void loadFromFile(File file) throws ADOMException {

    clear();
    xmlExplicit = null;

    try {

      DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      DocumentBuilder builder = factory.newDocumentBuilder();
      Document document = builder.parse(file);

      Element rootElm = document.getDocumentElement();
      ADOMNode rootNode = this;

      rootNode.setName(rootElm.getNodeName());

      convElmToNode(rootElm, rootNode);

    } catch (Exception ex) {
      throw new ADOMException("Unable to load ADOM from " + file.getAbsolutePath(), null, ex);
    }

  }

  /**
   * Parses the given XML string and creates the ADOMNodes for this ADOM object
   * @param xml The string that contains XML string to parse
   * @throws ADOMException if parse is not successful
   */
  public void parseXML(String xml) throws ADOMException {

    clear();
    xmlExplicit = null;

    try {

      DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      DocumentBuilder builder = factory.newDocumentBuilder();
      Document document = builder.parse( new ByteArrayInputStream(xml.getBytes()) );

      Element rootElm = document.getDocumentElement();
      ADOMNode rootNode = this;

      rootNode.setName(rootElm.getNodeName());

      convElmToNode(rootElm, rootNode);

    } catch (Exception ex) {
      throw new ADOMException("Unable to parse xml string ", null, ex);
    }

  }

}
