/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * ADOMBuilder.java
 *
 */

package org.moremotion.datasrc;

import org.moremotion.adom.ADOM;
import org.moremotion.adom.ADOMException;
import org.moremotion.config.ConfigurationException;
import org.moremotion.evaluator.Expression;
import org.moremotion.evaluator.MMSymbolResolver;
import org.moremotion.page.PageDataRequest;
import org.moremotion.security.AccessControlDefinition;
import org.moremotion.security.SecurityContext;
import org.moremotion.security.SecurityContextException;
import org.moremotion.security.SecurityManager;
import org.moremotion.security.User;
import org.moremotion.servlet.MoreMotionRequest;
import org.moremotion.util.Logger;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: ADOMBuilder.java 51 2008-03-10 11:46:54Z erkan $
 */
public class ADOMBuilder {
  
  private MoreMotionRequest request = null;
  private DataSourceConfig dsconfig = null;
  private String adomName = null;
  private char scope = ' ';
  private Logger logger = null;
  private boolean saveADOM = false;
  private PageDataRequest pdr = null;
  
  
  private void setLogger(Logger logger) {
    if (logger == null) {
      this.logger = new Logger(this);
    } else {
      this.logger = logger;
    }
  }
  
  /** Creates a new instance of ADOMBuilder */
  public ADOMBuilder(MoreMotionRequest request, DataSourceConfig dsconfig, Logger logger) {
    this.request = request;
    this.dsconfig = dsconfig;
    if (dsconfig != null)  this.adomName = dsconfig.getName();
    setLogger(logger);
  }
  
  public ADOMBuilder(MoreMotionRequest request, String dsname, Logger logger) throws Exception {
    this.request = request;
    this.adomName = dsname;
    this.dsconfig = DataSourceManager.getDataSourceConfig(request, dsname);
    setLogger(logger);
  }
  
  public void setAdomName(String adomName) {
    this.adomName = adomName;
    saveADOM = true;
  }
  
  public void setPageDataRequest(PageDataRequest pdr) {
    this.pdr = pdr;  
  }
  
  private ADOM getCurrentADOM() {
    
    switch (scope) {
      case 'r': return request.getRequestADOM(adomName);
      case 's': return request.getSessionADOM(adomName);
      case 'a': return request.getApplicationADOM(adomName);
    }
    
    ADOM adom = null;
    
    if (adom == null) adom = request.getRequestADOM(adomName);
    if (adom == null) adom = request.getSessionADOM(adomName);
    if (adom == null) adom = request.getApplicationADOM(adomName);
    
    return adom;
  }
  
  public ADOM build() throws ADOMException {
    
    try {
      
      this.scope = dsconfig == null ? ' ' : dsconfig.getScope();
      int refreshTimeout = (dsconfig == null) ? 0 : dsconfig.getRefreshTimeout(request);
      
      ADOM currentADOM = getCurrentADOM();
      
      if (logger.LEVEL1) logger.print("ADOM : " + currentADOM);
      
      /*
       * Is ADOM Stale?
       */
      boolean stale = (currentADOM != null && dsconfig != null && currentADOM.isStale(refreshTimeout));
      if (logger.LEVEL1) logger.print("Is ADOM stale?" + stale);
      
      /*
       * Needs Refresh?
       */
      boolean needsRefresh = dsconfig == null ? false : (new Paging(request,adomName,dsconfig)).getNeedsRefresh();
      if (logger.LEVEL1) logger.print("Needs Refresh? " + needsRefresh);
      
      boolean hasItemPerPage = false;
      
      hasItemPerPage = dsconfig != null && dsconfig.getItemsPerPage(request) > 0;
      
      if (currentADOM != null && !stale && !needsRefresh && !hasItemPerPage) {
        /*
         * ADOM does not need to be rebuilt. Just return the current ADOM.
         */
        return currentADOM;
      }
      
      if (logger.LEVEL1) logger.print("ADOM needs to be (re)built.");
      
      ADOM adom = null;
      
      String precond = dsconfig == null ? null : dsconfig.getPrecond(request);
      
      if (precond != null && precond.length() > 0) {
        
        if (logger.LEVEL1) logger.print("Testing precondition '" + precond + "'");
        
        MMSymbolResolver mmsr = request.getSymbolResolver();
        Expression expr = new Expression(precond, mmsr);
        
        try {
          if (!expr.evalBool()) {
            if (logger.LEVEL1) logger.print("Precondition not satisfied...");
            return null;
          }
        } catch (Exception e) {
          throw new ADOMException(e);
        }
        
      }
      
      /*
       * Check Access Authority
       */
      AccessControlDefinition acd = dsconfig == null ? null : dsconfig.getAccessControlDefinition(request);
      
      if (acd != null && acd.getCheckPoint() != null) {
        
        SecurityContext sctx;
        
        try {
          sctx = SecurityManager.getSecurityContext(request,acd.getSecurityDomain());
          
          User currentUser = sctx.getCurrentUser();
          if (currentUser == null || !sctx.hasAuthority(currentUser,acd.getCheckPoint())) {
            String msg = "Can't pass the checkpoint: " + acd.getCheckPoint() + " or user is not logged in!";
            if (logger.LEVEL1) logger.print("Build ADOM: " + msg);
            adom = new ADOM();
            adom.setNodeValue("exception", msg);
            return adom;
          }
          
        } catch (SecurityContextException sce) {
          if (sce.getMessage().equals("NO_SECURITY_COMPONENT_AVAILABLE")) {
            throw new ADOMException(sce);
          }
        }
        
      }
      
      if (scope == 's') {
////////////////////////////        // Bir nedeni olmal. Ama ne? !!!         
////////////////////////////        try {
////////////////////////////          if (currentADOM != null) currentADOM = (ADOM)currentADOM.clone();
////////////////////////////        } catch (Exception e) {
////////////////////////////          e.printStackTrace();
////////////////////////////        }
      } else {
        currentADOM = new ADOM();
      }
      
      adom = buildADOM(currentADOM);
      if (adom != null && dsconfig != null) {
        adom.setResBundleName( dsconfig.getResBundleName(request) );
      }
      
      return adom;
      
    } catch (ConfigurationException e) {
      throw new ADOMException(e);
    }
    
  }
  
  private ADOM buildADOM(ADOM currentADOM) throws ADOMException  {
    
    ADOM adom = null;
    
    String className = dsconfig == null ? null : dsconfig.getClassName();
    
    if (className != null) {
      
      if (logger.LEVEL1) logger.print("There is a class that can regenerate the ADOM.");
      
      if (currentADOM == null) {
        
        switch (scope) {
          case 'r': currentADOM = request.newRequestADOM( adomName ); break;
          case 's': currentADOM = request.newSessionADOM( adomName ); break;
          case 'a': currentADOM = request.newApplicationADOM( adomName ); break;
        }
        
      }
      
      try {
        
        DataServiceContext dsc = new DataServiceContext();
        dsc.setRequest(request);
        dsc.setDataSourceConfig(dsconfig);
        if (this.pdr != null) {
          dsc.setPageDataRequest(pdr);
        }
        
        Paging paging = new Paging(request,adomName,dsconfig);
        dsc.setPaging(paging);
        
        dsc.setLogger( dsconfig.getName(), dsconfig.getDebugLevel(request), dsconfig.getUnitName(request) );
        
        /*
         * Call Data Service
         */
        DataSourceManager.runDataService(className, currentADOM, dsc);
        
        /*
         * Add Paging information to ADOM
         */
        if (paging.getItemsPerPage() > 0) {
          paging.addPagingInfo(currentADOM);
        }
      } catch (Exception ex) {
        throw new ADOMException("DATA_SERVICE_ERROR",new String[]{dsconfig.getName()},ex);
      }
      
      adom = currentADOM;
      
      saveADOM(adom);
    }
    
    return adom;
  }
  
  private static final String ADOM_PREFIX = "ADOM_";
  
  private void saveADOM(ADOM adom) {
    
    if (!saveADOM) {
      return;
    }
    
    switch (scope) {
      case 'r': request.getHttpServletRequest().setAttribute(ADOM_PREFIX + adomName, adom); break;
      case 's': request.getSession().setAttribute(ADOM_PREFIX + adomName, adom); break;
      case 'a': request.getContext().setAttribute(ADOM_PREFIX + adomName, adom); break;
    }
    
  }
  
  
}
