/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * MScriptFunction.java
 *
 * Created on 10 November 2004 Wednesday, 15:49
 */

package org.moremotion.evaluator;


/**
 * Defines an Interface for custom MScript Functions.
 * <P>Custom MScript functions can be developed my implementing this interface.
 * </P><P>
 * When MScriptResolver encounters a function, i.e. <CODE>@repeat()</CODE> which is not built in, 
 * it first checks whether a function definition
 * exist in the configuration files. For Example: <BR/>
 * <PRE>
 *  &lt;mscriptFunction name="repeat"&gt;
 *  &nbsp; &lt;class&gt;mypack.RepeaterMScriptFunction&lt;/class&gt;
 *  &nbsp; &lt;repeat-seperator&gt;,&lt;/repeat-seperator&gt;
 *  &lt;/mscriptFunction&gt;
 * </PRE>
 * 
 * If it does, it loads the defined the class <CODE>mypack.RepeaterMScriptFunction</CODE>.
 * If it does not then it assumes that the function name is a class name and tries to load it. It there is a success
 * the resolver calls the <CODE>execute(Expression[] params, MScriptFunctionContext msfc)</CODE> 
 * method of the loaded class.
 * </P>
 * <P><B>An Example Implementation</B>
 * <PRE>
 * package mypack;

 * import org.moremotion.evaluator.*;
 * import org.moremotion.servlet.*;
 *   
 * public class RepeaterMScriptFunction implements MScriptFunction {
 *    
 *   public Object execute(Expression[] params, MScriptFunctionContext msfc) throws MScriptFunctionException {
 *     if (params.length &lt; 2) throw new Exception("Less parameters than expected");
 *     MMSymbolResolver sr = msfc.getSymbolResolver();
 *     String seperator = msfc.getMScriptFunctionConfig().getParameter("repeat-seperator").stringValue(null);
 *     String s = params[0].evaluate(sr).toString(); 
 *     int times = ((Double)params[1].evaluate(sr)).intValue();
 *     StringBuffer result = new StringBuffer();
 *     for (int i = 0; i &lt; times; i++) {
 *       if (result.length() &gt; 0) result.append(seperator);
 *       result.append(s);
 *     } 
 *     return result.toString();
 *   } 
 * }
 *
 * </PRE>
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: MScriptFunction.java 51 2008-03-10 11:46:54Z erkan $
 */
public interface MScriptFunction {


  /**
   * Executes the algorithm of the custom MScript function and returns the result.
   * @param params The function parameters as <code>Expression</code> object array.
   * @param msfc MScriptFunctionContext object
   * @return The result object. The expected objects are String, Double and Boolean.
   * @throws MScriptFunctionException when something goes wrong with the execution of the function.
   */
  public Object execute(Expression[] params, MScriptFunctionContext msfc) throws MScriptFunctionException;

}
