/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * SymbolResolver.java
 *
 * Created on 28 January 2003 Tuesday, 16:11
 */

package org.moremotion.evaluator;

/**
 * Defines an interface to provide the data types and the values of the symbols.
 * The objects of the classes that implement this interface can be introduced to
 * {@link org.moremotion.evaluator.Expression Expression} class to say that call My Symbol MScriptResolver for the
 * symbols existing in the expression.
 * <P><B>
 * Example:</B>
 * </P><PRE>
 *
 * import org.moremotion.evaluator.SymbolResolver;
 *
 * public class MySymbolResolver implements org.moremotion.evaluator.SymbolResolver {
 *
 *   int age = 0;
 *   boolean retired = false;
 *   String name = "John Coombes";
 *
 *   public MySymbolResolver() {
 *   }
 *
 *   public double getNumberValue(String symbol) throws Exception {
 *     if (symbol.equals("AGE")) return age;
 *     return 0;
 *   }
 *
 *   public boolean getBoolValue(String symbol) throws Exception {
 *     if (symbol.equals("RETIRED")) return retired;
 *     return false;
 *   }
 *
 *   public String getStringValue(String symbol) throws Exception {
 *     if (symbol.equals("NAME")) return name;
 *     return "";
 *   }
 *
 *   public int getType(String symbol) throws Exception {
 *     if (symbol.equals("AGE")) return DT_NUMBER;
 *     if (symbol.equals("RETIRED")) return DT_BOOL;
 *     return DT_STRING;
 *   }
 *
 *   public String getValue(String symbol) throws Exception {
 *     if (symbol.equals("AGE")) return "" + age;
 *     if (symbol.equals("RETIRED")) return "" + retired;
 *     if (symbol.equals("NAME")) return name;
 *     return "";
 *   }
 *
 *   public void setValue(String symbol, String value) throws Exception {
 *   }
 *
 * }
 * </PRE>
 * <P><B>Usage Example</B></P><PRE>
 * MySymbolResolver msr = new MySymbolResolver();
 * Expression expr = new Expression("A + (B / 3) - 2", msr);
 * String result = expr.eval();
 * </PRE>
 *
 *
 * @author Mehmet Tastan
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: SymbolResolver.java 137 2008-05-11 17:24:45Z erkan $
 */
public interface SymbolResolver {
  
  public static final byte
      DT_NULL       = -1, 
      DT_STRING     =  0, // "string""
      DT_NUMBER     =  1, // 123.45
      DT_BOOL       =  2; // true, false
  
  
  /**
   * Returns the type of the given symbol.
   * The types can be DT_STRING, DT_BOOL or DT_NUMBER
   */
  public int getType(String symbol) throws EvaluationException;
  
  /**
   * Returns the value of the given symbol as <CODE>String</CODE>.
   * This method will be called when type of the symbol is undefined.
   */
  public String getValue(String symbol) throws EvaluationException;
  
  /**
   * Returns the value of the given symbol as <CODE>String</CODE>.
   * This method will be called when type of the symbol is <CODE>String</CODE>.
   */
  public String getStringValue(String symbol) throws EvaluationException;
  
  /**
   * Returns the value of the given symbol as <CODE>double</CODE>.
   * This method will be called when type of the symbol is either <CODE>int</CODE> or <CODE>double</CODE>.
   */
  public double getNumberValue(String symbol) throws EvaluationException;
  
  /**
   * Returns the value of the given symbol as <CODE>boolean</CODE>.
   * This method will be called when type of the symbol is <CODE>boolean</CODE>.
   */
  public boolean getBoolValue(String symbol) throws EvaluationException;
  
  /**
   * Although this method takes place in this interface it will not be called by Expression class.
   */
  public void setValue(String symbol, String value) throws EvaluationException;
  
}
