/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

package org.moremotion.exception;

import java.sql.SQLException;
import org.moremotion.resource.Resource;
import org.moremotion.resource.ResourceException;
import org.moremotion.servlet.SystemConfig;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: MoreMotionException.java 51 2008-03-10 11:46:54Z erkan $
 */
public class MoreMotionException extends Exception {
  private String message = null;
  private String[] arguments = null;
  private String resBundleName = null;
  private Exception rootCause = null;
  
  /** Creates a new MoreMotionException.
   */
  public MoreMotionException(Exception rootCause) {
    super("Error!");
    this.message = "Error!";
    if (rootCause instanceof MoreMotionException) {
      MoreMotionException e = (MoreMotionException)rootCause;
      this.rootCause = e.getRootCause();
      this.rootCause = rootCause;
      this.resBundleName = e.getResBundleName();
      if (this.resBundleName == null) this.resBundleName = "moremotion";
    } else {
      this.rootCause = rootCause;
    }
  }
  
  /** Creates a new MoreMotionException.
   */
  public MoreMotionException(String message) {
    super(message);
    this.message = message;
    this.resBundleName = "moremotion";
  }
  
  /** Creates a new MoreMotionException.
   * @param message Exception message.
   * @param arguments message argument array.
   * @param rootCause The Exception that caused this Exception to be thrown
   * @param resBundleName resBundleName to get localized messages
   */
  public MoreMotionException(String message, String[] arguments, Exception rootCause, String resBundleName) {
    super(message);
    this.message = message;
    this.arguments = arguments;
    this.resBundleName = resBundleName;
    this.rootCause = rootCause;
  }
  
  /** Creates a new MoreMotionException. The resource bundle name is defaulted as "moremotion".
   * @param messsage Exception message.
   * @param arguments message argument array.
   * @param rootCause The Exception that caused this Exception to be thrown
   */
  public MoreMotionException(String messsage, String[] arguments, Exception rootCause) {
    this(messsage, arguments, rootCause, "moremotion");
  }
  
  /** Returns a localized message using resources.
   */
  public String getLocalizedMessage(String language) throws Exception {
    String message = this.message;
    try {
      if (resBundleName != null) {
        Resource r = new Resource(message,arguments);
        r.setBundleName(resBundleName);
        
        message = r.format(language);
      }
    } catch(ResourceException ignored) {}
    
    return message;
  }
  
  public String getResourceMessage(boolean resolve) throws Exception {
    String message = this.message;
    try {
      if (resBundleName != null) {
        Resource r = new Resource(message,getArguments());
        r.setBundleName(resBundleName);
        if (resolve) {
          message = r.format(SystemConfig.applicationLanguage());
        } else {
          message = r.toString();
        }
      }
    } catch(Exception ignored) {}
    
    return message;
  }
  
  /** Returns the message.
   */
  public String getMessage() {
    return this.message;
  }
  
  public String[] getArguments() {
    if (this.arguments == null && this.rootCause != null && this.rootCause instanceof MoreMotionException) {
      return ((MoreMotionException)rootCause).getArguments();
    }
    return this.arguments;
  }
  
  /** Returns root cause.
   */
  public Exception getRootCause() {
    return this.rootCause;
  }
  
  /** sets resource name.
   */
  public void setResBundleName(String resBundleName) {
    this.resBundleName = resBundleName;
  }
  
  /** Returns resource name.
   */
  public String getResBundleName() {
    return this.resBundleName;
  }
  
  /** Prints stack trace with the r
   * oot cause's stack trace.
   */
  public void printStackTrace() {
    super.printStackTrace();
    if (rootCause != null) {
      System.err.println("\nCaused by:");
      rootCause.printStackTrace();
    }
  }
  
  /** Prints stack trace with the root cause's stack trace.
   */
  public void printStackTrace(java.io.PrintStream ps) {
    // super.printStackTrace(ps);
    
    if (!this.message.equals("Error'")) {
      try {
        ps.println(getResourceMessage(true) + "\n");
        if (rootCause != null) ps.println("\nCaused by:");
      } catch(Exception e){};
    }
    
    if (rootCause != null) {
      rootCause.printStackTrace(ps);
    }
  }
  
  /** Prints stack trace with the root cause's stack trace.
   */
  public void printStackTrace(java.io.PrintWriter pw) {
    if (!this.message.equals("Error!")) {
      try {
        pw.println(getResourceMessage(true) + "\n");
        if (rootCause != null) pw.println("\nCaused by:");
      } catch(Exception e){};
    }
    
    if (rootCause != null) {
      rootCause.printStackTrace(pw);
    }
  }
  
  /**
   * Returns the original error code that caused the exception.
   */
  public java.sql.SQLException getOriginalSQLException() {
    if (rootCause == null) {
      return null;
    }
    if (this.rootCause instanceof java.sql.SQLException) {
      return (java.sql.SQLException)this.rootCause;
    }
    if (rootCause instanceof MoreMotionException) {
      return ((MoreMotionException)rootCause).getOriginalSQLException();
    }
    return null;
  }
  
  
}