/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */


package org.moremotion.page;

import org.moremotion.config.ConditionalList;
import org.moremotion.config.ConfigParameter;
import org.moremotion.config.ConfigParameterException;
import org.moremotion.security.AccessControlConfigParameter;
import org.moremotion.security.AccessControlDefinition;
import org.moremotion.servlet.MoreMotionRequest;
import org.moremotion.servlet.SystemConfig;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Contains information about a page such as language, encoding, checkpoint, alternative pages, allowed commands, etc.
 *
 * @author  Yusuf Boyacigil
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: PageConfig.java 51 2008-03-10 11:46:54Z erkan $
 */
public class PageConfig {
  
  private String name = null;
  private String language = "en";
  
  private boolean ajaxSourcePage = false;
  
  private ConfigParameter encoding = null;
  
  private ConfigParameter avoidCaching = null;
  
  private AccessControlConfigParameter accessControl = null;
  
  private PageDataRequestList dataRequests = null;
  private ConditionalList commandDefs = null;
  
  private ConditionalList templateCalls = null;
  /**
   * Creates new PageConfig
   */
  protected PageConfig(String pageName, Document doc) throws PageGenerationException {
    this.name = pageName;
    try {
      parse(doc);
    } catch (Exception e) {
      throw new PageGenerationException(e);
    }
  }
  
  public String getName() {
    return name;
  }
  
  public boolean isAjaxSourcePage() {
    return this.ajaxSourcePage;
  }
  
  private void parse(Document doc) throws Exception {
    
    NodeList nlist = doc.getDocumentElement().getChildNodes();
    for (int i = 0; i < nlist.getLength(); i++) {
      
      Node node = nlist.item(i);
      
      if (node.getNodeType() == Node.ELEMENT_NODE) {
        
        String nodeName = node.getNodeName();
        
        if (nodeName.equals("language")) {
          language = node.getFirstChild().getNodeValue();
          if (language.equals("applicationLanguage")) {
            language = SystemConfig.applicationLanguage();
          }
        }
        
        else if (nodeName.equals("encoding")) {
          encoding = new ConfigParameter("encoding", node.getFirstChild().getNodeValue() );
        }
        
        else if (nodeName.equals("avoidCaching")) {
          avoidCaching = new ConfigParameter("avoidCaching", node.getFirstChild().getNodeValue() );
        }
        
        else if (nodeName.equals("accessControl")) {
          accessControl = new AccessControlConfigParameter((Element)node);
        }
        
        else if (nodeName.equals("ajaxSourcePage")) {
          this.ajaxSourcePage = node.getFirstChild().getNodeValue().equalsIgnoreCase("true");
        }
        
        else if (nodeName.equals("dataRequest")) {
          
          if (dataRequests == null) {
            dataRequests = new PageDataRequestList();
          }
          dataRequests.add( new PageDataRequest(this,(Element)node) );
          
        }
        
        else if (nodeName.equals("command")) {
          if (commandDefs == null) {
            commandDefs = new ConditionalList();
          }
          commandDefs.add( new CommandDefinition((Element)node) );
        }
        
        else if (nodeName.equals("processTemplateCall")) {
          if (templateCalls == null) {
            templateCalls = new ConditionalList();
          }
          templateCalls.add( new TemplateCall((Element)node, this.name) );
        }
        
      }
      
    }
    
    
  }
  
  /**
   * Returns the Data Requests of the page. 
   */
  public PageDataRequestList getDataRequests() {
    return this.dataRequests;
  }
  
  /**
   * Returns the Template Process Calls of the page. 
   */
  public ConditionalList getProcessTemplateCalls() {
    return this.templateCalls;
  }
  
  /**
   * Returns AccessControlDefinition object of the page
   */
  public AccessControlDefinition getAccessControlDefinition(MoreMotionRequest request) throws ConfigParameterException {
    return accessControl == null ? null : accessControl.getAccessControlDefinition(request);
  }
  
  
  private String safe(String str, String def) {
    if (str == null || str.length() == 0) {
      return def;
    }
    return str;
  }
  
  
  /**
   * Returns the language code
   */
  public String getLanguage() {
    if (language.length() == 0) {
      return "en";
    }
    return language;
  }
  
  /**
   * Returns the page encoding
   */
  public String getEncoding(MoreMotionRequest request) throws ConfigParameterException {
    return encoding == null ? "iso-8859-1" : encoding.resolve(request).stringValue("iso-8859-1");
  }

  /**
   * Returns <CODE>true</CODE> if Avoid Caching is specified in the page config, <CODE>false</CODE> otherwise
   */
  public boolean getAvoidCaching(MoreMotionRequest request) throws ConfigParameterException {
    return avoidCaching == null ? false : avoidCaching.resolve(request).booleanValue(false);
  }
  
  
  /**
   * Returns the command definitions
   */
  public ConditionalList getCommandDefinitions() throws PageGenerationException {
    return this.commandDefs;
  }
  
}
