/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * CallDefinition.java
 *
 * Created on 09 January 2006 Monday , 16:55
 */

package org.moremotion.process;

import java.util.ArrayList;
import java.util.Iterator;
import org.moremotion.config.ConditionalListItem;
import org.moremotion.config.ConfigCache;
import org.moremotion.config.ConfigNodeList;
import org.moremotion.config.ConfigParameter;
import org.moremotion.config.ConfigParameterException;
import org.moremotion.config.ConfigUnit;
import org.moremotion.config.ConfigurationException;
import org.moremotion.evaluator.Expression;
import org.moremotion.servlet.MoreMotionRequest;
import org.moremotion.util.XString;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Represents the Call Definition belonging to one of "callProcess", "callQueryProcess" or "callTemplate"
 * process configuration elements.
 *
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: CallDefinition.java 67 2008-03-17 07:03:46Z erkan $
 */
public abstract class CallDefinition extends ConditionalListItem {
  
  private Object parent = null;
  private String targetName;
  private String stepName = null;
  
  private ConfigParameter blockName;
  private ConfigParameter recordFilter;
  private ConfigParameter debugLevel;
  private ConfigParameter reset = null;
  private ArrayList params = null;
  private ProcessConfig pconfig = null;  
  
  /* This will help to recognise when a CallDefinition has an
   * updated ConfigUnit
   */
  private ConfigUnit configUnitLink = null;
  
  
  public CallDefinition(Element elm, Object parent) throws ConfigurationException  {
    super(elm);
    this.parent = parent;
    if (elm != null) {
      parse(elm);
    } else {
      emptyDefinition();
    }
  }
  
  private void emptyDefinition() {
    
    blockName = new ConfigParameter("blockName",null);
    recordFilter = new ConfigParameter("recordFilter",null);
    
    debugLevel = new ConfigParameter("debugLevel",null);
    reset = new ConfigParameter("reset",null);
    
  }
  
  private void parse(Element elm) throws ConfigurationException {
    
    targetName = elm.getAttribute("name");
    if (targetName.length() == 0) {
      throw new ConfigurationException("TARGET_NAME_MISSING");
    }
    
    stepName = elm.getAttribute("stepName");
    if (stepName.length() == 0) stepName = null;
    
    blockName = new ConfigParameter("blockName",elm.getAttribute("blockName"));
    recordFilter = new ConfigParameter("recordFilter",elm.getAttribute("recordFilter"));
    
    debugLevel = new ConfigParameter("debugLevel",elm.getAttribute("debugLevel"));
    reset = new ConfigParameter("reset",elm.getAttribute("reset"));
    
    NodeList nlist = elm.getElementsByTagName("param");
    for (int i = 0; i < nlist.getLength(); i++) {
      Node node = nlist.item(i);
      if (params == null) params = new ArrayList();
      params.add(node);
    }
    
  }
  
  protected void setTargetName(String name) {
    this.targetName = name;
  }
  
  protected ArrayList getParams() {
    return this.params;
  }
  
  public String getPageName() {
    Object po = this;
    while (po instanceof CallDefinition && ((CallDefinition)po).parent != null) {
      po = ((CallDefinition)po).parent;
    }
    return ((ProcessTemplateConfig)po).pageName;
  }
  
  public String getTargetName() {
    return targetName;
  }
  
  public String getStepName() {
    return stepName == null ? targetName : stepName;
  }
  
  public String getPureStepName() {
    return stepName;
  }
  
  /**
   * Returns a caption for this call definition. It returns the step name if defined, process name otherwise.
   */
  public String getStepCaption() {
    
    Object o = this;
    String caption = "";
    
    while (o != null && o instanceof CallDefinition) {
      String s = ((CallDefinition)o).stepName;
      if (s == null || s.equals("")) s = ((CallDefinition)o).targetName;
      if (caption.length() > 0) caption = "::" + caption;
      caption = s + caption;
      o = ((CallDefinition)o).parent;
    }
    
    if (o != null && o instanceof ProcessTemplateConfig) {
      caption = ((ProcessTemplateConfig)o).getName() + "::" + caption;
    }
    return caption;
  }
  
  public ProcessConfig getProcessConfig() {
    return this.pconfig;
  }
  
  /**
   * Returns the name of the input process block of the target process
   */
  public String getBlockName(MoreMotionRequest request) throws ConfigParameterException {
    return blockName.resolve(request).stringValue(null);
  }
  
  public Expression getRecordFilter() {
    return recordFilter.expressionValue(null);
  }
  
  public int getDebugLevel(MoreMotionRequest request) throws ConfigParameterException {
    return debugLevel.resolve(request).intValue(-1);
  }
  
  public String getReset(MoreMotionRequest request) throws ConfigParameterException { 
    return reset.resolve(request).stringValue(null);
  }
  
  private String getParentName() {
    if (parent == null) return "";
    if (parent instanceof ProcessTemplateConfig) {
      return ((ProcessTemplateConfig)parent).getName();
    } else {
      return ((CallDefinition)parent).targetName;
    }
  }
  
  
  protected void prepareProcessConfig() throws ConfigurationException {

    ConfigUnit unit = null;

    String page = getPageName();
    if (this.targetName.charAt(0) != '/' && page != null) {
      unit = ConfigCache.getUnit("process", page + targetName);
    }
    if (unit == null) {
      unit = ConfigCache.getUnit("process",XString.after(targetName,'/'));
    }

    if (unit == null) {
      throw new ConfigurationException("PROCESS_CONFIG_MISSING",new String[]{getTargetName()},null);
    }

    pconfig = new ProcessConfig(unit);

    if (params != null) {
      pconfig.parse(getInplaceParams() );
    }

    unit.setDataObject("*");
    this.configUnitLink = unit;
  }


  private ConfigNodeList getInplaceParams() {

    ConfigNodeList list = new ConfigNodeList();

    Iterator itr = params.iterator();

    while (itr.hasNext()) {
      Node node = (Node)itr.next();
      list.addLast(node);
    }

    return list;
  }  
  
  public String toString() {
    
    StringBuffer buf = new StringBuffer();
    
    buf.append("[ parent: " + getParentName());
    buf.append(", targetName: " + targetName);
    buf.append(", stepName: " + stepName);
    buf.append(", blockName: " + blockName.stringValue(null));
    buf.append(", recordFilter: " + recordFilter.stringValue(null));
    buf.append(", debugLevel: " + debugLevel);
    buf.append(", reset: " + reset);
    buf.append(", params : " + (params == null ? "0" : "" + params.size()) + " params");
    buf.append("]");
    
    return buf.toString();
  }
  
}

